#  ---------------------------------------------------------------------------
#  pyCGNS - Python package for CFD General Notation System - 
#  See license.txt file in the root directory of this Python module source  
#  ---------------------------------------------------------------------------
#
from __future__ import unicode_literals
from builtins import (str, bytes, range, dict)

import CGNS
import CGNS.PAT.cgnskeywords as CK
import CGNS.PAT.cgnstypes as CT
import CGNS.PAT.cgnserrors as CE
import CGNS.PAT.cgnsutils as CU

import numpy as NPY


# -----------------------------------------------------------------------------
class CGNSPythonChildren(list):
    def __getitem__(self, key):
        if isinstance(key, str):
            for c in self:
                if c.name == key:
                    return c
        else:
            return list.__getitem__(self, key)


# -----------------------------------------------------------------------------
class CGNSPython(object):
    """
    A CGNS/Python object
    """

    def __init__(self, node, parent=None):
        self.__node = node
        self.__parent = parent

    @property
    def name(self):
        """
        Name of the node (node[0])
        """
        return self.__node[0]

    @property
    def sidstype(self):
        return self.__node[3]

    @property
    def sids(self):
        return self.sidstype

    @property
    def value(self):
        return self.data

    @property
    def data(self):
        return self.__node[1]

    @property
    def child(self):
        l = CGNSPythonChildren([CGNSPython(n) for n in self.__node[2]])
        for n in l:
            n.parent = self
        return l

    @property
    def children(self):
        return self.__node[2]

    def nextChild(self, sidstype=None, namepattern=None):
        for c in self.__node[2]:
            take = False
            if sidstype is not None:
                if ((isinstance(sidstype, list) and (c[3] in sidstype)) or
                        (isinstance(sidstype, str) and (c[3] == sidstype))):
                    take = True
            else:
                take = True
            if take:
                n = CGNSPython(c)
                n.parent = self
                yield n

    @property
    def node(self):
        return self.__node

    @property
    def parent(self):
        return self.__parent

    @parent.setter
    def parent(self, node):
        if isinstance(node, CGNSPython):
            self.__parent = node
        else:
            self.__parent = CGNSPython(node)

    def __str__(self):
        return CU.toString(self.__node)

    def __len__(self):
        return len(self.__node[2])

# ---



#  -------------------------------------------------------------------------
#  pyCGNS.VAL - Python package for CFD General Notation System - 
#  See license.txt file in the root directory of this Python module source  
#  -------------------------------------------------------------------------
from __future__ import unicode_literals
from __future__ import print_function
import CGNS.VAL.simplecheck


def check(t, trace=False, grammar=None, paths=[''], stop=False,
          warnings=[], failures=[]):
    if grammar is None:
        grammar = ['SIDS']
    chk = CGNS.VAL.simplecheck.compliant(t, userlist=grammar, paths=paths, stop=stop,
                                         warnings=warnings, failures=failures)
    if trace:
        for d in chk[1]:
            print('# %s >>> %s' % (d[1], d[0]))
    return chk[0]



#  -------------------------------------------------------------------------
#  pyCGNS - Python package for CFD General Notation System - 
#  See license.txt file in the root directory of this Python module source  
#  -------------------------------------------------------------------------
#
from __future__ import unicode_literals
from __future__ import print_function
from builtins import (str, bytes, range, dict)

def asQuery(f):
    def prepostquery(node, parent, tree, links, skips, path, args, selected):
        import CGNS.PAT.cgnskeywords as CGK
        import CGNS.PAT.cgnsutils as CGU
        import CGNS.PAT.cgnslib as CGL
        import CGNS.MAP as CGM
        import numpy

        QueryNoException = True

        class Context(object):
            pass

        C = Context()
        C.NODE = node
        C.PARENT = parent
        C.NAME = node[0]
        C.VALUE = node[1]
        C.CGNSTYPE = node[3]
        C.CHILDREN = node[2]
        C.TREE = tree
        C.PATH = path
        C.LINKS = links
        C.SKIPS = skips
        C.USER = args
        C.ARGS = args
        C.SELECTED = selected
        C.RESULT_LIST = [False]

        if (QueryNoException):
            RESULT = f(C)
        else:
            try:
                RESULT = f(C)
                if (RESULT not in [True, False]): RESULT = False
                RESULT_LIST[0] = RESULT
            except Exception:
                RESULT_LIST[0] = False

        return RESULT

    return prepostquery


# -----------------------------------------------------------------
def parseAndSelect(tree, node, parent, links, skips, path, query, args, selected,
                   result):
    path = path + '/' + node[0]
    print('ARGS ', args)
    Q = query(node, parent, tree, links, skips, path, args, selected)
    R = []
    if (Q):
        if (result):
            R = [Q]
        else:
            R = [path]
    for C in node[2]:
        R += parseAndSelect(tree, C, node, links, skips, path, query, args, selected,
                            result)
    return R


# -----------------------------------------------------------------
def runQuery(tree, links, paths, query, args, selected=[], mode=True):
    """Recursively applies a function on all nodes of a tree, breadth-first
    parse, results returned in a list (breadth-first order of the list).

    tree,links,paths: the three restults returned by the CGNS.MAP.load call
    query: the query funciton as a name (string) or a callable object (such as
    a function)
    args: a tuple of args to pass to the query (for example a node name to
    look for)
    selected: the RETURNED list of values
    mode: True (defaut) a boolean list is returned for every node in the tree,
    False a list of the (True) paths is returned
    
    The match between results and paths can be performed with help of the
    breadth-first paths order function in CGNS.PAT
    """
    v = None
    try:
        try:
            if (args): v = eval(args)
        except TypeError:
            v = args
        if ((v is not None) and (type(v) not in [tuple, list])): v = (v,)
    except NameError:
        v = (str(args),)
    except:
        pass
    _args = v
    if (type(query) in [str,]): query = eval(query)
    result = parseAndSelect(tree, tree, [None, None, [], None], links, paths, '',
                            query, _args, selected, mode)
    return result



#  ---------------------------------------------------------------------------
#  pyCGNS - Python package for CFD General Notation System -
#  See license.txt file in the root directory of this Python module source  
#  ---------------------------------------------------------------------------
#
from __future__ import unicode_literals
from __future__ import absolute_import
import CGNS.PAT.cgnslib as C
import CGNS.PAT.cgnserrors as E
import CGNS.PAT.cgnskeywords as K
import numpy as N
import copy
#
from . import BCData_t

#
data = C.newBCDataSet(None, '{BCDataSet}')
C.newGridLocation(data)
C.newPointRange(data)
C.newPointList(data)
C.newDescriptor(data, '{Descriptor}')
C.newDataClass(data)
C.newDimensionalUnits(data)
C.newReferenceState(data)
C.newUserDefinedData(data, '{UserDefinedData}')
#
d1 = copy.deepcopy(BCData_t.pattern[0])
d1[0] = K.NeumannData_s
data[2].append(d1)
#
d2 = copy.deepcopy(BCData_t.pattern[0])
d2[0] = K.DirichletData_s
data[2].append(d2)
#
status = '9.4'
comment = 'Full SIDS with all optionals'
pattern = [data, status, comment]
#



#  -------------------------------------------------------------------------
#  pyCGNS.VAL - Python package for CFD General Notation System - VALidater
#  See license.txt file in the root directory of this Python module source  
#  -------------------------------------------------------------------------
#
from __future__ import unicode_literals
import CGNS.PAT.cgnslib as CGL
import CGNS.PAT.cgnsutils as CGU
import CGNS.PAT.cgnskeywords as CGK
import numpy as NPY

TESTS = []

#  -------------------------------------------------------------------------
tag = 'zonesubregion'
diag = True


def makeCorrectTree(vertexsize, cellsize, ntris):
    T = CGL.newCGNSTree()
    b = CGL.newBase(T, 'Base', 3, 3)
    s = NPY.array([[vertexsize, cellsize, 0]], dtype='int32', order='F')
    z = CGL.newZone(b, 'Zone', s, CGK.Unstructured_s)
    g = CGL.newGridCoordinates(z, 'GridCoordinates')
    d = CGL.newDataArray(g, CGK.CoordinateX_s, NPY.ones((vertexsize), dtype='float64', order='F'))
    d = CGL.newDataArray(g, CGK.CoordinateY_s, NPY.ones((vertexsize), dtype='float64', order='F'))
    d = CGL.newDataArray(g, CGK.CoordinateZ_s, NPY.ones((vertexsize), dtype='float64', order='F'))
    tetras = CGL.newElements(z, 'TETRAS', CGK.TETRA_4_s,
                             NPY.array([[1, cellsize]], 'i', order='F'),
                             NPY.ones((cellsize * 4), dtype='int32'))
    tris = CGL.newElements(z, 'TRIS', CGK.TRI_3_s,
                           NPY.array([[cellsize + 1, cellsize + ntris]],'i',order='F'),
                           NPY.ones((ntris * 3), dtype='int32'))
    zbc = CGL.newZoneBC(z)
    n = CGL.newBoundary(zbc, 'BC', [range(cellsize + 1, cellsize + ntris + 1)], btype=CGK.Null_s, family=None,
                        pttype=CGK.PointList_s)
    g = CGL.newGridLocation(n, value=CGK.FaceCenter_s)
    return (T, b, z)


vertexsize = 20
cellsize = 7
ntris = 11
(T, b, z) = makeCorrectTree(vertexsize, cellsize, ntris)
zsr1 = CGU.newNode('SubRegion1', NPY.array([3], dtype=NPY.int32, order='F'), [], CGK.ZoneSubRegion_ts, parent=z)
CGU.newNode(CGK.PointRange_s, NPY.array([[1, vertexsize]], dtype=NPY.int32, order='F'), [], CGK.IndexRange_ts,
            parent=zsr1)
CGL.newDataArray(zsr1, 'var', value=NPY.ones(vertexsize, dtype='float64', order='F'))
zsr2 = CGU.newNode('SubRegion2', NPY.array([3], dtype=NPY.int32, order='F'), [], CGK.ZoneSubRegion_ts, parent=z)
CGU.newNode(CGK.PointList_s, NPY.array([range(1, vertexsize + 1)], dtype=NPY.int32, order='F'), [],
            CGK.IndexArray_ts, parent=zsr2)
CGL.newDataArray(zsr2, 'var', value=NPY.ones(vertexsize, dtype='float64', order='F'))
zsr3 = CGU.newNode('SubRegion3', NPY.array([2], dtype=NPY.int32, order='F'), [], CGK.ZoneSubRegion_ts, parent=z)
CGU.newNode(CGK.BCRegionName_s, CGU.setStringAsArray('BC'), [], CGK.Descriptor_ts, parent=zsr3)
CGL.newDataArray(zsr3, 'var', value=NPY.ones(ntris, dtype='float64', order='F'))
zsr5 = CGU.newNode('SubRegion5', NPY.array([3], dtype=NPY.int32, order='F'), [], CGK.ZoneSubRegion_ts, parent=z)
CGU.newNode(CGK.PointRange_s, NPY.array([[1, cellsize]], dtype=NPY.int32, order='F'), [], CGK.IndexRange_ts,
            parent=zsr5)
CGL.newGridLocation(zsr5, value=CGK.CellCenter_s)
CGL.newDataArray(zsr5, 'var', value=NPY.ones(cellsize, dtype='float64', order='F'))
zsr6 = CGU.newNode('SubRegion6', NPY.array([3], dtype=NPY.int32, order='F'), [], CGK.ZoneSubRegion_ts, parent=z)
CGU.newNode(CGK.PointList_s, NPY.array([range(1, cellsize + 1)], dtype=NPY.int32, order='F'), [],
            CGK.IndexArray_ts, parent=zsr6)
CGL.newGridLocation(zsr6, value=CGK.CellCenter_s)
CGL.newDataArray(zsr6, 'var', value=NPY.ones(cellsize, dtype='float64', order='F'))
TESTS.append((tag, T, diag))

#  -------------------------------------------------------------------------
tag = 'zonesubregion bad node value'
diag = False
(T, b, z) = makeCorrectTree(vertexsize, cellsize, ntris)
zsr1 = CGU.newNode('SubRegion1', NPY.array([4], dtype=NPY.int32, order='F'), [], CGK.ZoneSubRegion_ts, parent=z)
CGU.newNode(CGK.PointRange_s, NPY.array([[1, vertexsize]], dtype=NPY.int32, order='F'), [], CGK.IndexRange_ts,
            parent=zsr1)
TESTS.append((tag, T, diag))

#  -------------------------------------------------------------------------
tag = 'zonesubregion bad Grid Location'
diag = False
(T, b, z) = makeCorrectTree(vertexsize, cellsize, ntris)
zsr1 = CGU.newNode('SubRegion1', NPY.array([2], dtype=NPY.int32, order='F'), [], CGK.ZoneSubRegion_ts, parent=z)
CGU.newNode(CGK.PointRange_s, NPY.array([[1, cellsize]], dtype=NPY.int32, order='F'), [], CGK.IndexRange_ts,
            parent=zsr1)
CGL.newGridLocation(zsr1, value=CGK.CellCenter_s)
TESTS.append((tag, T, diag))

#  -------------------------------------------------------------------------
tag = 'zonesubregion bad children'
diag = False
(T, b, z) = makeCorrectTree(vertexsize, cellsize, ntris)
zsr1 = CGU.newNode('SubRegion1', NPY.array([3], dtype=NPY.int32, order='F'), [], CGK.ZoneSubRegion_ts, parent=z)
CGU.newNode(CGK.PointRange_s, NPY.array([[1, vertexsize]], dtype=NPY.int32, order='F'), [], CGK.IndexRange_ts,
            parent=zsr1)
CGU.newNode(CGK.PointList_s, NPY.array([range(1, vertexsize + 1)], dtype=NPY.int32, order='F'), [],
            CGK.IndexArray_ts, parent=zsr1)
CGU.newNode(CGK.BCRegionName_s, CGU.setStringAsArray('bc'), [], CGK.Descriptor_ts, parent=zsr1)
TESTS.append((tag, T, diag))

#  -------------------------------------------------------------------------
tag = 'zonesubregion bad regionname'
diag = False
(T, b, z) = makeCorrectTree(vertexsize, cellsize, ntris)
zsr1 = CGU.newNode('SubRegion1', NPY.array([2], dtype=NPY.int32, order='F'), [], CGK.ZoneSubRegion_ts, parent=z)
CGU.newNode(CGK.BCRegionName_s, CGU.setStringAsArray('bc'), [], CGK.Descriptor_ts, parent=zsr1)
zsr4 = CGU.newNode('SubRegion4', NPY.array([2], dtype=NPY.int32, order='F'), [], CGK.ZoneSubRegion_ts, parent=z)
CGU.newNode(CGK.GridConnectivityRegionName_s, CGU.setStringAsArray('GC'), [], CGK.Descriptor_ts, parent=zsr4)
TESTS.append((tag, T, diag))

#  -------------------------------------------------------------------------
tag = 'zonesubregion bad dataarray shapes'
diag = False
(T, b, z) = makeCorrectTree(vertexsize, cellsize, ntris)
zsr1 = CGU.newNode('SubRegion1', NPY.array([3], dtype=NPY.int32, order='F'), [], CGK.ZoneSubRegion_ts, parent=z)
CGU.newNode(CGK.PointRange_s, NPY.array([[1, vertexsize]], dtype=NPY.int32, order='F'), [], CGK.IndexRange_ts,
            parent=zsr1)
CGL.newDataArray(zsr1, 'var', value=NPY.ones(vertexsize + 1, dtype='float64', order='F'))
zsr2 = CGU.newNode('SubRegion2', NPY.array([3], dtype=NPY.int32, order='F'), [], CGK.ZoneSubRegion_ts, parent=z)
CGU.newNode(CGK.PointList_s, NPY.array([range(1, vertexsize + 1)], dtype=NPY.int32, order='F'), [],
            CGK.IndexArray_ts, parent=zsr2)
CGL.newDataArray(zsr2, 'var', value=NPY.ones(vertexsize + 1, dtype='float64', order='F'))
zsr3 = CGU.newNode('SubRegion3', NPY.array([2], dtype=NPY.int32, order='F'), [], CGK.ZoneSubRegion_ts, parent=z)
CGU.newNode(CGK.BCRegionName_s, CGU.setStringAsArray('BC'), [], CGK.Descriptor_ts, parent=zsr3)
CGL.newDataArray(zsr3, 'var', value=NPY.ones(cellsize, dtype='float64', order='F'))
TESTS.append((tag, T, diag))

#  -------------------------------------------------------------------------
tag = 'zonesubregion bad point list on cell and face subregions'
diag = False
(T, b, z) = makeCorrectTree(vertexsize, cellsize, ntris)
zsr6 = CGU.newNode('SubRegion6', NPY.array([3], dtype=NPY.int32, order='F'), [], CGK.ZoneSubRegion_ts, parent=z)
CGU.newNode(CGK.PointList_s, NPY.array([range(1, cellsize + 2)], dtype=NPY.int32, order='F'), [],
            CGK.IndexArray_ts, parent=zsr6)
CGL.newGridLocation(zsr6, value=CGK.CellCenter_s)
zsr7 = CGU.newNode('SubRegion7', NPY.array([3], dtype=NPY.int32, order='F'), [], CGK.ZoneSubRegion_ts, parent=z)
CGU.newNode(CGK.PointList_s, NPY.array([range(cellsize, cellsize + ntris)], dtype=NPY.int32, order='F'), [],
            CGK.IndexArray_ts, parent=zsr7)
CGL.newGridLocation(zsr7, value=CGK.FaceCenter_s)
TESTS.append((tag, T, diag))

#  -------------------------------------------------------------------------
tag = 'zonesubregion bad point range on cell and face subregions'
diag = False
(T, b, z) = makeCorrectTree(vertexsize, cellsize, ntris)
zsr5 = CGU.newNode('SubRegion5', NPY.array([3], dtype=NPY.int32, order='F'), [], CGK.ZoneSubRegion_ts, parent=z)
CGU.newNode(CGK.PointRange_s, NPY.array([[1, cellsize + 1]], dtype=NPY.int32, order='F'), [], CGK.IndexRange_ts,
            parent=zsr5)
CGL.newGridLocation(zsr5, value=CGK.CellCenter_s)
zsr8 = CGU.newNode('SubRegion8', NPY.array([3], dtype=NPY.int32, order='F'), [], CGK.ZoneSubRegion_ts, parent=z)
CGU.newNode(CGK.PointRange_s, NPY.array([[cellsize, cellsize + ntris]], dtype=NPY.int32, order='F'), [],
            CGK.IndexRange_ts, parent=zsr8)
CGL.newGridLocation(zsr8, value=CGK.FaceCenter_s)
TESTS.append((tag, T, diag))



#  -------------------------------------------------------------------------
#  pyCGNS - Python package for CFD General Notation System - 
#  See license.txt file in the root directory of this Python module source  
#  -------------------------------------------------------------------------
# 
"""
 implementation info:

   All Q7***Window(s) are created using Qt Designer,
   each produces a Ui_Q7***Window as Cython class,
   which is wrapped by a Q7*** python class.
   This latter is the one to use with Python.
   Same with stuff like Widgets.

   The Q7fingerPrint is the actual class with all CGNS tree info,
   including the list of views and selections. It is more or less
   the model itself but with some of local stuff and I want two
   separate classes.
"""
from __future__ import unicode_literals

def show(T, *args):
    from CGNS.NAV.moption import Q7OptionContext as OCTXT
    import qtpy.QtCore as QtCore
    from qtpy.QtWidgets import QApplication, QSplashScreen
    from qtpy.QtGui import QPixmap
    from CGNS.NAV.wcontrol import Q7Main

    from CGNS.NAV.wfile import Q7File
    from CGNS.NAV.winfo import Q7Info
    from CGNS.NAV.woption import Q7Option
    from CGNS.NAV.wtree import Q7Tree
    from CGNS.NAV.mtree import Q7TreeModel
    from CGNS.NAV.wfingerprint import Q7FingerPrint
    from CGNS.NAV.wquery import Q7Query
    from CGNS.NAV.whelp import Q7Help

    def dummy(*args):
        pass

    app = QApplication(args)

    app.updateViews = dummy
    app.removeTreeStatus = dummy
    app.addTreeStatus = dummy
    app.loadOptions = dummy
    app.addLine = dummy
    app.delLine = dummy
    app.transientRecurse = False
    app.transientVTK = False

    app.verbose = True
    app._application = app
    app.wOption = Q7Option(app)
    app.wOption.reset()
    app.processEvents()

    fgprint = Q7FingerPrint(app, '.', '<run-time>', T, [], [])
    Q7TreeModel(fgprint.index)
    child = Q7Tree(app, '/', fgprint.index)

    for wg in [child.bSave, child.bQueryView, child.bSaveAs, child.bInfo,
               child.bCheck, child.bCheckList, child.bClearChecks,
               child.bPatternView, child.bToolsView, child.bFormView,
               child.bSelectLinkSrc, child.bSelectLinkDst,
               child.bBackControl]:
        wg.setEnabled(False)

    fgprint._status = [Q7FingerPrint.STATUS_MODIFIED]
    child.show()

    app.exec_()

# --- last line



#  -------------------------------------------------------------------------
#  pyCGNS - Python package for CFD General Notation System - 
#  See license.txt file in the root directory of this Python module source  
#  -------------------------------------------------------------------------
#
from __future__ import unicode_literals
from builtins import (str, bytes, range, dict)

from qtpy.QtCore import (Qt, QObject, Signal)
from qtpy.QtWidgets import (QAction, QStyledItemDelegate, QStyleOptionViewItem,
                            QMenu, QTableWidgetItem, QHeaderView)

from CGNS.NAV.moption import Q7OptionContext as OCTXT

import CGNS.config as config

import CGNS.PAT.cgnslib as CGL

from CGNS.NAV.Q7ControlWindow import Ui_Q7ControlWindow
from CGNS.NAV.wfile import Q7File
from CGNS.NAV.winfo import Q7Info
from CGNS.NAV.woption import Q7Option
from CGNS.NAV.wfingerprint import Q7FingerPrint
from CGNS.NAV.wquery import Q7Query
from CGNS.NAV.whelp import Q7Help

import CGNS.NAV.wmessages as MSG

from CGNS.NAV.wfingerprint import Q7Window as QW


# -----------------------------------------------------------------
class Q7SignalPool(QObject):
    loadFile = Signal(name='loadFile')
    saveFile = Signal(name='saveFile')
    cancel = Signal(name='cancel')
    loadCompleted = Signal(name='loadCompleted')
    buffer = None
    fgprint = None
    saveAs = False


# -----------------------------------------------------------------
class Q7ControlItemDelegate(QStyledItemDelegate):
    def paint(self, painter, option, index):
        if index.column() in [0, 1]:
            option.decorationPosition = QStyleOptionViewItem.Top
            QStyledItemDelegate.paint(self, painter, option, index)
        else:
            QStyledItemDelegate.paint(self, painter, option, index)


# -----------------------------------------------------------------
class Q7Main(QW, Ui_Q7ControlWindow):
    verbose = False

    def __init__(self, parent=None):
        self.lastView = None
        self.w = None
        self.fdialog = None
        import platform
        QW.control_log = MSG.Q7Log()
        QW.__init__(self, QW.VIEW_CONTROL, self, None, None)
        self.versions = {'pycgnsversion': 'pyCGNS v%s' % config.version,
                         #                       'chloneversion':'CHLone %s'%config.CHLONE_VERSION,
                         'vtkversion': 'VTK v%s' % config.VTK_VERSION,
                         'cythonversion': 'Cython v%s' % config.CYTHON_VERSION,
                         'hdf5version': 'HDF5 v%s' % config.HDF5_VERSION,
                         'numpyversion': 'numpy v%s' % config.NUMPY_VERSION,
                         'pythonversion': 'python v%s' % platform.python_version(),
                         'pyqtversion': 'PyQt v%s' % config.PYQT_VERSION,
                         'qtversion': 'Qt v%s' % config.QT_VERSION,
                         }
        self.getHistory()
        self.bAbout.clicked.connect(self.about)
        self.bOptionView.clicked.connect(self.option)
        self.bTreeLoadLast.clicked.connect(self.loadlast)
        self.lockable(self.bTreeLoadLast)
        self.bTreeLoad.clicked.connect(self.load)
        self.bLog.clicked.connect(self.logView)
        self.lockable(self.bTreeLoad)
        self.bEditTree.clicked.connect(self.edit)
        self.lockable(self.bEditTree)
        self.bInfo.clicked.connect(self.infoControl)
        self.bPatternView.setDisabled(True)
        # self.bResetScrollBars.clicked.connect(self.resetScrolls)
        self.bClose.clicked.connect(self.close)
        # QObject.connect(self.controlTable,
        #                SIGNAL("cellClicked(int,int)"),
        #                self.clickedLine)
        self.controlTable.cellClicked.connect(self.clickedLine)
        self.initControlTable()
        self.controlTable.setItemDelegate(Q7ControlItemDelegate(self))
        self.signals = Q7SignalPool()
        self.signals.loadFile.connect(self.loadStart)
        self.signals.saveFile.connect(self.saving)
        self.signals.cancel.connect(self.cancelUnlock)
        self.signals.loadCompleted.connect(self.loadCompleted)
        self.setContextMenuPolicy(Qt.CustomContextMenu)
        self.popupmenu = QMenu()
        self.transientRecurse = False
        self.transientVTK = False
        self.copyPasteBuffer = None
        self.wOption = None
        self.selectForLinkDst = None
        self.newtreecount = 1
        self.help = None
        self._patternwindow = None
        self._toolswindow = None
        self.query = None
        Q7Query.loadUserQueries()
        Q7Query.fillQueries()
        Q7Query.loadUserFunctions()

    def clickedLine(self, *args):
        if self.controlTable.lastButton == Qt.LeftButton:
            # Q7FingerPrint.raiseView(self.getIdxFromLine(args[0]))
            pass
        if self.controlTable.lastButton == Qt.RightButton:
            self.updateMenu(self.controlTable.currentIndex())
            self.popupmenu.popup(self.controlTable.lastPos)

    def closeView(self):
        self.updateLastView()
        if self.lastView is not None:
            fg = Q7FingerPrint.getFingerPrint(self.lastView)
            fg.closeView(self.lastView)
            self.lastView = None

    def raiseView(self):
        self.updateLastView()
        if self.lastView is not None:
            Q7FingerPrint.raiseView(self.lastView)

    def logView(self):
        self.control_log.show()

    def infoControl(self):
        self.helpWindow('Control')

    def helpWindowDoc(self, doc):
        if self.help is not None:
            self.help.close()
        self.help = Q7Help(self, doc=doc)
        self.help.show()

    def helpWindow(self, key):
        if self.help is not None:
            self.help.close()
        self.help = Q7Help(self, key)
        self.help.show()

    def info(self):
        self.updateLastView()
        if self.lastView is not None:
            (f, v, d) = Q7FingerPrint.infoView(self.lastView)
            if not f.isFile():
                return
            self.w = Q7Info(self, d, f)
            self.w.show()

    def closeTree(self):
        self.updateLastView()
        if self.lastView is None:
            return
        (f, v, d) = Q7FingerPrint.infoView(self.lastView)
        reply = MSG.wQuestion(self, 101, 'Double check...',
                              """Do you want to close the tree and all its views,<br>
                              and <b>forget unsaved</b> modifications?""")
        if reply:
            f.closeAllViews()

    def closeAllTrees(self):
        reply = MSG.wQuestion(self, 101, 'Double check...',
                              """Do you want to close all the views,<br>
                              and <b>forget unsaved</b> modifications?""")
        if reply:
            Q7FingerPrint.closeAllTrees()

    def updateLastView(self):
        it = self.controlTable.currentItem()
        if it is None:
            self.lastView = None
        else:
            r = it.row()
            self.lastView = self.getIdxFromLine(r)
        return self.lastView

    def updateMenu(self, idx):
        lv = self.getIdxFromLine(idx.row())
        if lv is not None:
            self.lastView = lv
            actlist = (("View information (Enter)", self.info),
                       ("Raise selected view (Space)", self.raiseView),
                       None,
                       ("Close all trees", self.closeAllTrees),
                       ("Close selected tree", self.closeTree),
                       ("Close selected view (Del)", self.closeView))
            self.popupmenu.clear()
            self.popupmenu.setTitle('Control view menu')
            for aparam in actlist:
                if aparam is None:
                    self.popupmenu.addSeparator()
                else:
                    a = QAction(aparam[0], self)
                    a.triggered.connect(aparam[1])
                    self.popupmenu.addAction(a)

    def loadOptions(self):
        if self.wOption is None:
            self.wOption = Q7Option(self)
        self.wOption.reset()

    def option(self):
        if self.wOption is None:
            self.wOption = Q7Option(self)
        self.wOption.show()

    def about(self):
        MSG.wInfo(self, 100, "pyCGNS v%s" % OCTXT._ToolVersion,
                  OCTXT._CopyrightNotice % self.versions, again=False)

    def closeApplication(self):
        reply = MSG.wQuestion(self, 101, 'Double check...',
                              """Do you want to quit %s,<b>close all views</b>
                              and forget unsaved modifications?"""
                              % OCTXT._ToolName)
        if reply == MSG.OK:
            Q7FingerPrint.closeAllTrees()
            if self.help is not None:
                self.help.close()
            if self._patternwindow is not None:
                self._patternwindow.close()
            if self.control_log is not None:
                self.control_log.close()
            if self._toolswindow is not None:
                self._toolswindow.close()
            return True
        else:
            return False

    def closeEvent(self, event):
        if self.closeApplication():
            event.accept()
        # return True
        else:
            event.ignore()
            #            return False

    def resetScrolls(self):
        self.controlTable.verticalScrollBar().setSliderPosition(0)
        self.controlTable.horizontalScrollBar().setSliderPosition(0)

    def initControlTable(self):
        ctw = self.controlTable
        ctw.control = self
        cth = ctw.horizontalHeader()
        ctw.verticalHeader().hide()
        h = ['S', 'T', 'View', 'Dir', 'File', 'Node']
        for i in range(len(h)):
            hi = QTableWidgetItem(h[i])
            hi.setFont(OCTXT._Label_Font)
            ctw.setHorizontalHeaderItem(i, hi)
            cth.setSectionResizeMode(i, QHeaderView.ResizeToContents)
        cth.setSectionResizeMode(len(h) - 1, QHeaderView.Stretch)

    def updateViews(self):
        for i in self.getAllIdx():
            f = Q7FingerPrint.getFingerPrint(i)
            v = Q7FingerPrint.getView(i)
            l = self.getLineFromIdx(i)
            self.modifiedLine(l, f._status, f)
            try:
                v.updateTreeStatus()
            except AttributeError:
                pass

    def modifiedLine(self, n, stat, fg):
        if ((Q7FingerPrint.STATUS_MODIFIED in stat) and
                (Q7FingerPrint.STATUS_SAVEABLE in stat)):
            stitem = QTableWidgetItem(self.IC(QW.I_MOD_SAV), '')
            stitem.setToolTip('Tree modified and saveable')
        if ((Q7FingerPrint.STATUS_MODIFIED in stat) and
                (Q7FingerPrint.STATUS_SAVEABLE not in stat)):
            stitem = QTableWidgetItem(self.IC(QW.I_MOD_USAV), '')
            stitem.setToolTip('Tree modified but NOT saveable')
        if ((Q7FingerPrint.STATUS_MODIFIED not in stat) and
                (Q7FingerPrint.STATUS_SAVEABLE not in stat)):
            stitem = QTableWidgetItem(self.IC(QW.I_UMOD_USAV), '')
            stitem.setToolTip('Tree unmodified and NOT saveable')
        if ((Q7FingerPrint.STATUS_MODIFIED not in stat) and
                (Q7FingerPrint.STATUS_SAVEABLE in stat)):
            stitem = QTableWidgetItem(self.IC(QW.I_UMOD_SAV), '')
            stitem.setToolTip('Tree unmodified and saveable')
        stitem.setTextAlignment(Qt.AlignCenter)
        self.controlTable.setItem(n, 0, stitem)
        self.controlTable.item(n, 3).setText(str(fg.filedir))
        self.controlTable.item(n, 4).setText(str(fg.filename))

    def addLine(self, l, fg):
        ctw = self.controlTable
        ctw.setRowCount(ctw.rowCount() + 1)
        r = ctw.rowCount() - 1
        if l[1] == QW.VIEW_TREE:
            tpitem = QTableWidgetItem(self.IC(QW.I_TREE), '')
        if l[1] == QW.VIEW_FORM:
            tpitem = QTableWidgetItem(self.IC(QW.I_FORM), '')
        if l[1] == QW.VIEW_VTK:
            tpitem = QTableWidgetItem(self.IC(QW.I_VTK), '')
        if l[1] == QW.VIEW_QUERY:
            tpitem = QTableWidgetItem(self.IC(QW.I_QUERY), '')
        if l[1] == QW.VIEW_SELECT:
            tpitem = QTableWidgetItem(self.IC(QW.I_SELECT), '')
        if l[1] == QW.VIEW_DIAG:
            tpitem = QTableWidgetItem(self.IC(QW.I_DIAG), '')
        if l[1] == QW.VIEW_TOOLS:
            tpitem = QTableWidgetItem(self.IC(QW.I_TOOLS), '')
            l = l[0:2] + [None, None, None]
        if l[1] == QW.VIEW_LINK:
            tpitem = QTableWidgetItem(self.IC(QW.I_LINK), '')
        if l[1] == QW.VIEW_DIFF:
            tpitem = QTableWidgetItem(self.IC(QW.I_DIFF), '')
        tpitem.setTextAlignment(Qt.AlignCenter)
        ctw.setItem(r, 1, tpitem)
        for i in range(len(l) - 2):
            it = QTableWidgetItem('%s ' % (l[i + 2]))
            if i in [0]:
                it.setTextAlignment(Qt.AlignCenter)
            else:
                it.setTextAlignment(Qt.AlignLeft)
            it.setFont(OCTXT._Table_Font)
            ctw.setItem(r, i + 2, it)
        self.modifiedLine(r, l[0], fg)
        ctw.setColumnWidth(0, 25)
        ctw.setColumnWidth(1, 25)
        ctw.setColumnWidth(2, 50)
        for i in range(self.controlTable.rowCount()):
            ctw.resizeRowToContents(i)

    def selectLine(self, idx):
        i = int(self.getLineFromIdx(idx))
        if i != -1:
            self.controlTable.setCurrentCell(i, 2)

    def delLine(self, idx):
        i = int(self.getLineFromIdx(idx))
        if i != -1:
            self.controlTable.removeRow(i)

    def getIdxFromLine(self, l):
        self.controlTable.setCurrentCell(l, 2)
        it = self.controlTable.currentItem()
        return it.text()

    def getLineFromIdx(self, idx):
        found = -1
        for n in range(self.controlTable.rowCount()):
            if int(idx) == int(self.controlTable.item(n, 2).text()):
                found = n
        return found

    def getAllIdx(self):
        all = []
        for n in range(self.controlTable.rowCount()):
            all.append(self.controlTable.item(n, 2).text())
        return all

    def clearOtherSelections(self):
        if self._patternwindow is not None:
            self._patternwindow.clearSelection()

    def cancelUnlock(self, *args):
        self.lockView(False)

    def loadStart(self, *args):
        self._T('loading: [%s]' % self.signals.buffer)
        self.busyCursor()
        Q7FingerPrint.treeLoad(self, self.signals.buffer)
        Q7FingerPrint.refreshScreen()

    def setDefaults(self):
        self.loadOptions()
        self._application.setStyleSheet(self.wOption._options['UserCSS'])

    def loadCompleted(self, *args, **kwargs):
        self.lockView(False)
        if 'dataset_name' in kwargs:
            filedir = kwargs['dataset_base']
            filename = kwargs['dataset_name']
            tree = kwargs['dataset_tree']
            links = kwargs['dataset_references']
            paths = kwargs['dataset_paths']
            fgprint = Q7FingerPrint(self, filedir, filename,
                                    tree, links, paths, **kwargs)
        else:
            fgprint = self.signals.fgprint
        if len(fgprint) > 1:
            code = fgprint[1][0]
            msg0 = fgprint[1][1]            
            msg1 = fgprint[1][2]
            MSG.wError(self, code, msg0, msg1)
        elif fgprint.tree is None:
            MSG.wError(self, 201, 'Load error',
                       'Fatal error while loading file, empty tree')
        else:
            child = self.loadQ7Tree(fgprint)
            child.show()
            self.setHistory(fgprint.filedir, fgprint.filename)
            self.updateViews()
            fgprint.getInfo(force=True)
        self.signals.fgprint = None
        Q7FingerPrint.killProxy()
        self.readyCursor()

    def saving(self, *args):
        self._T('saving as: [%s]' % self.signals.buffer)
        self.busyCursor()
        Q7FingerPrint.treeSave(self, self.signals.fgprint, self.signals.buffer,
                               self.signals.saveAs)
        self.setHistory(self.signals.fgprint.filedir,
                        self.signals.fgprint.filename)
        self.updateViews()
        self.signals.fgprint.getInfo(force=True)
        self.readyCursor()
        self.lockView(False)

    def load(self):
        self.fdialog = Q7File(self)
        self.lockView(True)
        self.fdialog.show()

    def loadlast(self):
        if self.getLastFile() is None:
            return
        self.signals.buffer = self.getLastFile()[0] + '/' + self.getLastFile()[1]
        if self.signals.buffer is None:
            self.load()
        else:
            self.signals.loadFile.emit()

    def loadfile(self, name):
        self.signals.buffer = name
        self.signals.loadFile.emit()

    def save(self, fgprint):
        self.signals.fgprint = fgprint
        self.signals.saveAs = True
        self.fdialog = Q7File(self, 1)
        self.lockView(True)
        self.fdialog.show()

    def savedirect(self, fgprint):
        self.signals.fgprint = fgprint
        self.signals.saveAs = False
        self.signals.buffer = fgprint.filedir + '/' + fgprint.filename
        self.signals.saveFile.emit()

    def edit(self):
        self._T('edit new')
        tree = CGL.newCGNSTree()
        tc = self.newtreecount
        self.busyCursor()
        fgprint = Q7FingerPrint(self, '.', 'new#%.3d.hdf' % tc, tree, [], [])
        child = self.loadQ7Tree(fgprint)
        fgprint._status = [Q7FingerPrint.STATUS_MODIFIED]
        self.readyCursor()
        self.newtreecount += 1
        child.show()

    def userFunctionFromPath(self, path, types):
        return Q7Query._userFunction

    def loadQ7Tree(self, fgprint):
        from CGNS.NAV.wtree import Q7Tree
        from CGNS.NAV.mtree import Q7TreeModel
        Q7TreeModel(fgprint.index)
        return Q7Tree(self, '/', fgprint.index)

# -----------------------------------------------------------------



#  -------------------------------------------------------------------------
#  pyCGNS - Python package for CFD General Notation System - 
#  See license.txt file in the root directory of this Python module source  
#  -------------------------------------------------------------------------
#
from __future__ import unicode_literals
from builtins import (str, bytes, range, dict)

from CGNS.NAV.moption import Q7OptionContext as OCTXT

import sys
import numpy
import os
import importlib
import glob

import CGNS.PAT.cgnsutils as CGU
import CGNS.PAT.cgnskeywords as CGK
import CGNS.PAT.SIDS

from qtpy.QtCore import Qt
from qtpy.QtWidgets import (QFileDialog, QTableWidgetItem, QStyledItemDelegate)
from qtpy.QtGui import QFont

from CGNS.NAV.Q7PatternWindow import Ui_Q7PatternWindow
from CGNS.NAV.wfingerprint import Q7Window as QW

# -----------------------------------------------------------------
class Q7PatternList(QW, Ui_Q7PatternWindow):
    def __init__(self, control, fgprint):
        QW.__init__(self, QW.VIEW_PATTERN, control, None, None)
        self.bClose.clicked.connect(self.reject)
        self.bSave.clicked.connect(self.patternsave)
        self.bInfo.clicked.connect(self.infoPatternView)
        self.bCopy.clicked.connect(self.copyPatternInBuffer)
        self.bDelete.clicked.connect(self.deletelocal)
        self.bSave.setDisabled(True)
        self.bDelete.setDisabled(True)
        self.bAdd.setDisabled(True)
        self._profiles = {}
        self._profiles['SIDS'] = CGNS.PAT.SIDS.profile
        self.loadUserProfiles()
        self._modified = False
        self._initialized = False
        self._selected = None
        self._master = control
        self.patternTable._panel = self

    def infoPatternView(self):
        self._control.helpWindow('Pattern')

    def clearSelection(self):
        if self._selected is not None:
            rold = self.findRow(*self._selected)
            if rold != -1:
                it1 = QTableWidgetItem(self.IC(QW.I_EMPTY), '')
                self.patternTable.setItem(rold, 0, it1)

    def findRow(self, pit, nit):
        maxrow = self.patternTable.rowCount()
        for row in range(maxrow):
            rnit = self.patternTable.item(row, 1).text()
            rpit = self.patternTable.item(row, 2).text()
            if (rpit, rnit) == (pit, nit):
                return row
        return -1

    def copyPatternInBuffer(self):
        ix = self.patternTable.currentIndex()
        if not ix.isValid():
            return
        row = ix.row()
        nit = self.patternTable.item(ix.row(), 1).text()
        pit = self.patternTable.item(ix.row(), 2).text()
        if (pit, nit) == self._selected:
            self.clearSelection()
            self._selected = None
            return
        self.clearSelection()
        self._selected = (pit, nit)
        self._control.copyPasteBuffer = self._profiles[pit][nit][0]
        it1 = QTableWidgetItem(self.IC(QW.I_MARK), '')
        self.patternTable.setItem(row, 0, it1)

    def show(self):
        if not self._initialized:
            self.reset()
        super(Q7PatternList, self).show()
        self.raise_()

    def patternsave(self):
        filename = QFileDialog.getSaveFileName(self,
                                               "Save pattern", ".", "*.py")
        if filename[0] == "":
            return
        # f=open(str(filename[0]),'w+')
        # f.write(n)
        # f.close()

    def deletelocal(self):
        pass

    def reset(self):
        tlvcols = 4
        tlvcolsnames = ['S', 'Pattern', 'P', 'Comment']
        v = self.patternTable
        v.setColumnCount(tlvcols)
        lh = v.horizontalHeader()
        lv = v.verticalHeader()
        h = tlvcolsnames
        n = len(h)
        for i in range(n):
            hi = QTableWidgetItem(h[i])
            v.setHorizontalHeaderItem(i, hi)
        for profkey in self._profiles:
            prof = self._profiles[profkey]
            for k in prof:
                pentry = prof[k]
                v.setRowCount(v.rowCount() + 1)
                r = v.rowCount() - 1
                it1 = QTableWidgetItem(self.IC(QW.I_EMPTY), '')
                it2 = QTableWidgetItem(k)
                it2.setFont(QFont("Courier"))
                it3 = QTableWidgetItem(profkey)
                it4 = QTableWidgetItem(pentry[2])
                v.setItem(r, 0, it1)
                v.setItem(r, 1, it2)
                v.setItem(r, 2, it3)
                v.setItem(r, 3, it4)
        self.patternTable.resizeColumnsToContents()
        self.patternTable.resizeRowsToContents()
        plist = []
        for i in range(len(plist)):
            v.resizeColumnToContents(i)
        for i in range(v.rowCount()):
            v.resizeRowToContents(i)
        self._initialized = True

    def reject(self):
        if self._master._patternwindow is not None:
            self._master._patternwindow = None
        self.close()

    def loadUserProfiles(self):
        pthlist = OCTXT.ProfileSearchPathList
        if not pthlist:
            return
        pthlistok = []
        for pth in pthlist:
            if os.path.isdir(pth):
                pthlistok += [pth]
        if not pthlistok:
            return
        for pth in pthlistok:
            sys.path.append(pth)
            modlist = [os.path.splitext(os.path.basename(mod))[0]
                       for mod in glob.glob('%s/*' % pth)]
            for m in modlist:
                pym = importlib.import_module(m)
                self._profiles[m] = pym.profile


# -----------------------------------------------------------------
class Q7PatternTableItemDelegate(QStyledItemDelegate):
    def paint(self, painter, option, index):
        t = index.data(Qt.DisplayRole)
        r = option.rect.adjusted(2, 2, -2, -2)
        painter.drawText(r, Qt.AlignVCenter | Qt.AlignLeft, t, r)

# -----------------------------------------------------------------



#  -------------------------------------------------------------------------
#  pyCGNS.VAL - Python package for CFD General Notation System - VALidater
#  See license.txt file in the root directory of this Python module source  
#  -------------------------------------------------------------------------
#
from __future__ import unicode_literals
from __future__ import print_function
import CGNS.PAT.cgnsutils      as CGU
import CGNS.PAT.cgnstypes      as CGT
import CGNS.PAT.cgnskeywords   as CGK
import CGNS.PAT.cgnserrors     as CGE
import CGNS.VAL.parse.messages as CGM

import inspect

genericmessages = {
    'G0001': (CGM.CHECK_FAIL, 'CGNSLibraryVersion [%s] is too old wrt check level'),
    'G0002': (CGM.CHECK_FAIL, 'CGNSLibraryVersion is incorrect'),
    'G0003': (CGM.CHECK_FAIL, 'Name [%s] is not valid'),
    'G0004': (CGM.CHECK_FAIL, 'Name [%s] is a duplicated child name'),
    'G0005': (CGM.CHECK_FAIL, 'PANIC: Cannot find node with path [%s]'),
    'G0006': (CGM.CHECK_FAIL, 'PANIC: Node data is not numpy.ndarray or None'),
    'G0007': (CGM.CHECK_FAIL, 'PANIC: Node children is not a list'),
    'G0008': (CGM.CHECK_FAIL, 'PANIC: Node name is not a string'),
    'G0009': (CGM.CHECK_FAIL, 'PANIC: Node is not a list of 4 objects'),
    'G0010': (CGM.CHECK_FAIL, 'PANIC: Node is empty list or None'),
    'G0011': (CGM.CHECK_FAIL, 'PANIC: Node name is empty string'),
    'G0012': (CGM.CHECK_FAIL, 'PANIC: Node name has forbidden chars'),
    'G0013': (CGM.CHECK_FAIL, 'PANIC: Node name is . or ..'),
    'G0014': (CGM.CHECK_FAIL, 'PANIC: Node name is too long'),
    'G0015': (CGM.CHECK_FAIL, 'Bad node value data type'),
    'S0001': (CGM.CHECK_FAIL, 'Unknown SIDS type [%s]'),
    'S0002': (CGM.CHECK_FAIL, 'SIDS type [%s] not allowed as child of [%s]'),
    'S0003': (CGM.CHECK_FAIL, 'SIDS type [%s] not allowed for this node'),
    'S0004': (CGM.CHECK_FAIL, 'DataType [%s] not allowed for this node'),
    'S0005': (CGM.CHECK_FAIL, 'Node [%s] of type [%s] not allowed as child'),
    'S0006': (CGM.CHECK_FAIL, 'Node [%s] of type [%s] allowed only once as child'),
    'S0007': (CGM.CHECK_FAIL, 'Node [%s] of type [%s] is mandatory'),
    'S0008': (CGM.CHECK_FAIL, 'Child name [%s] reserved for a type in [%s]'),
    'S0009': (CGM.CHECK_FAIL, 'Bad node shape [%s]'),
    'S0010': (CGM.CHECK_FAIL, 'Bad node value'),
}


class PathContext(dict):
    def __getitem__(self, path):
        if path not in self:
            return self.scope(path)
        return dict.__getitem__(self, path)

    def scope(self, path):
        if path in self:
            return dict.__getitem__(self, path)
        level = 1
        while True:
            apath = CGU.getPathAncestor(path, level)
            if apath in self:
                return dict.__getitem__(self, apath)
            if apath == '/':
                return None
            level += 1


class GenericContext(dict):
    def __getitem__(self, key):
        if (key not in self):
            dict.__setitem__(self, key, PathContext())
        return dict.__getitem__(self, key)

    def __setitem__(self, key, value):
        d = self[key]
        d['/'] = value


class GenericParser(object):
    # --------------------------------------------------------------------
    def __init__(self, log=None):
        self._stop = False
        self.keywordlist = CGK.cgnsnames
        if log is None:
            self.log = CGM.DiagnosticLog()
        self.log.addMessages(genericmessages)
        self.context = GenericContext()
        self._trace = False

    # --------------------------------------------------------------------
    def dbg(self, msg, *args):
        if self._trace:
            print('> %-24s' % msg, )
            if args:
                for a in args:
                    print(a, )
            print("")

    # --------------------------------------------------------------------
    def listDiagnostics(self):
        return self.log.listMessages()

    # --------------------------------------------------------------------
    def checkLeafStructure(self, T, path, node, parent):
        stt = CGM.CHECK_GOOD
        try:
            CGU.checkNode(node, dienow=True)
            CGU.checkNodeName(node, dienow=True)
            CGU.checkDuplicatedName(parent, node[0], dienow=True)
            CGU.checkNodeType(node, dienow=True)
            if (node[1] is not None):
                CGU.checkArray(node[1], dienow=True)
        except CGE.cgnsException as v:
            if (v.code == 1):
                stt = self.log.push(path, 'G0010')
            if (v.code == 2):
                stt = self.log.push(path, 'G0009')
            if (v.code == 3):
                stt = self.log.push(path, 'G0008')
            if (v.code == 4):
                stt = self.log.push(path, 'G0007')
            if (v.code == 5):
                stt = self.log.push(path, 'G0006')
            if (v.code == 22):
                stt = self.log.push(path, 'G0008')
            if (v.code == 23):
                stt = self.log.push(path, 'G0011')
            if (v.code == 24):
                stt = self.log.push(path, 'G0012')
            if (v.code == 25):
                stt = self.log.push(path, 'G0014')
            if (v.code == 29):
                stt = self.log.push(path, 'G0013')
            if (v.code == 31):
                stt = self.log.push(path, 'G0011')
            if (v.code == 32):
                stt = self.log.push(path, 'G0011')
            if (v.code == 111):
                stt = self.log.push(path, 'G0015')
            if (self._stop):
                raise CGE.cgnsException(-1)
        return stt

    # --------------------------------------------------------------------
    def checkLeaf(self, T, path, node):
        parent = CGU.getParentFromNode(T, node)
        status1 = self.checkSingleNode(T, path, node, parent)
        status2 = status1
        ntype = CGU.getTypeAsGrammarToken(node[3])
        if (len(node) == 4) and (ntype in self.methods):
            status2 = getattr(self, ntype)(*[path, node, parent, T, self.log])
        else:
            # if (ntype in CGK.cgnstypes): print '\nSKIP ',ntype
            pass
        status1 = CGM.getWorst(status1, status2)
        if self._stop and status1 == CGM.CHECK_FAIL:
            raise CGE.cgnsException(-1)
        return status1

    # --------------------------------------------------------------------
    def checkSingleNode(self, T, path, node, parent):
        stt = CGM.CHECK_GOOD
        if (not CGU.checkNodeName(node)):
            stt = self.log.push(path, 'G0003', node[0])
        lchildren = CGU.childNames(parent)
        if (lchildren):
            if (node[0] in lchildren):
                lchildren.remove(node[0])
            if (node[0] in lchildren):
                stt = self.log.push(path, 'G0004', node[0])
        tlist = CGU.getNodeAllowedChildrenTypes(parent, node)
        if ((CGU.getTypeAsGrammarToken(node[3]) not in tlist) and
                (node[3] != CGK.CGNSTree_ts)):
            if (parent is not None):
                stt = self.log.push(path, 'S0002', node[3], parent[3])
            else:
                stt = self.log.push(path, 'S0003', node[3])
        dlist = CGU.getNodeAllowedDataTypes(node)
        dt = CGU.getValueDataType(node)
        if (dt not in dlist):
            stt = self.log.push(path, 'S0004', dt)
        if (node[3] not in CGT.types.keys()):
            stt = self.log.push(path, 'S0001', node[3])
        else:
            stt = self.checkCardinalityOfChildren(T, path, node, parent)
            stt = self.checkReservedChildrenNames(T, path, node, parent)
        return stt

    # --------------------------------------------------------------------
    def checkTreeStructure(self, T, path=''):
        status = CGM.CHECK_GOOD
        status = self.checkLeafStructure(T, path, T, None)
        if (status == CGM.CHECK_GOOD):
            path = path + '/' + T[0]
            for c in T[2]:
                status = self.checkTreeStructure(c, path)
        return status

    # --------------------------------------------------------------------
    def checkTree(self, T, trace=False, stop=False):
        self._stop = stop
        self._trace = trace
        status1 = CGM.CHECK_GOOD
        if (self._trace):
            print('### Parsing node paths...')
        status1 = self.checkTreeStructure(T)
        if (status1 != CGM.CHECK_GOOD):
            return status1
        paths = CGU.getPathFullTree(T, width=True)
        sz = len(paths) + 1
        ct = 1
        if (not hasattr(self, 'methods')):
            self.methods = []
            for m in inspect.getmembers(self):
                if ((m[0][-2:] == '_t') or (m[0][-2:] == '_n') or (m[0][-3:] == '_ts')):
                    self.methods += [m[0]]
        for path in ['/'] + paths:
            if (self._trace):
                print('### Check node [%.6d/%.6d]\r' % (ct, sz), )
            node = CGU.getNodeByPath(T, path)
            status2 = CGM.CHECK_GOOD
            if (node is None):
                status2 = self.log.push(path, 'G0005', path)
                if (self._stop):
                    raise CGE.cgnsException(-1)
            if (status2 == CGM.CHECK_GOOD):
                status2 = self.checkLeaf(T, path, node)
            status1 = status2
            ct += 1
        if self._trace:
            print("")
        return status1

    # --------------------------------------------------
    def checkCardinalityOfChildren(self, T, path, node, parent):
        stt = CGM.CHECK_GOOD
        for child in node[2]:
            card = CGT.types[node[3]].cardinality(child[3])
            if (card == CGT.C_00):
                if (path == '/'):
                    cpath = '/%s' % (child[0])
                else:
                    cpath = '%s/%s' % (path, child[0])
                stt = self.log.push(path, 'S0005', cpath, child[3])
            if (card in [CGT.C_11, CGT.C_01]):
                if ([c[3] for c in node[2]].count(child[3]) > 1):
                    stt = self.log.push(path, 'S0006', child[0], child[3])
        for tchild in CGT.types[node[3]].children:
            card = CGT.types[node[3]].cardinality(tchild[0])
            if (card in [CGT.C_11, CGT.C_1N]):
                if ([c[3] for c in node[2]].count(tchild[0]) < 1):
                    stt = self.log.push(path, 'S0007', tchild[1][0], tchild[0])
        return stt

    # --------------------------------------------------
    def checkReservedChildrenNames(self, T, path, node, parent):
        stt = CGM.CHECK_GOOD
        for child in node[2]:
            rt = CGT.types[node[3]].hasReservedNameType(child[0])
            if ((rt != []) and (child[3] not in rt)):
                srt = ""
                for s in rt:
                    srt = srt + "," + s
                stt = self.log.push(path, 'S0008', child[0], srt[1:])
        return stt

    # --------------------------------------------------------------------
    def CGNSLibraryVersion_t(self, pth, node, parent, tree, log):
        stt = CGM.CHECK_OK
        try:
            version = int(node[1][0] * 1000)
            if version < 2400:
                stt = self.log.push(pth, 'G0001', version)
            if version > 3400:
                stt = self.log.push(pth, 'G0002')
        except Exception:
            stt = self.log.push(pth, 'G0002')
        return stt

# --- last line



#  -------------------------------------------------------------------------
#  pyCGNS - Python package for CFD General Notation System - 
#  See license.txt file in the root directory of this Python module source  
#  -------------------------------------------------------------------------
#
from __future__ import unicode_literals
from builtins import (str, bytes, range, dict)

import CGNS.PAT.cgnskeywords as CGK

tlistA = [
    CGK.Descriptor_ts,
    CGK.UserDefinedData_ts,
    CGK.DataClass_ts,
    CGK.DimensionalUnits_ts,
]

allDT = [CGK.C1, CGK.MT, CGK.I4, CGK.I8, CGK.R4, CGK.R8]  # LK is default

C_00 = 'Zero/Zero'
C_01 = 'Zero/One'
C_11 = 'One/One'
C_0N = 'Zero/N'
C_1N = 'One/N'
C_NN = 'N/N'

UD = '{UserDefined}'

allCARD = [C_01, C_11, C_0N, C_1N, C_NN]


# --------------------------------------------------------
class CGNStype:
    def __init__(self, ntype, dtype=None, names=None):
        if dtype is None:
            dtype = [CGK.MT]
        if names is None:
            names = [UD]
        self.type = ntype
        self.datatype = [CGK.LK] + dtype
        self.enumerate = []
        self.shape = ()
        self.names = names
        self.children = []
        self.parents = []

    def hasChild(self, ctype):
        for c in self.children:
            if c[0] == ctype:
                return True
        return False

    def addChild(self, ctype, cname=UD, dtype=CGK.MT, card=C_0N):
        if type(cname) != list:
            lname = [cname]
        else:
            lname = cname
        self.children.append((ctype, lname, dtype, card))

    def addParent(self, parent):
        self.parents.append(parent)

    def cardinality(self, childtype):
        for c in self.children:
            if c[0] == childtype:
                return c[3]
        return C_00

    def isReservedName(self, name):
        for c in self.children:
            if name in c[1]:
                return True
        return False

    def hasReservedNameType(self, name):
        nl = []
        for c in self.children:
            if name in c[1]:
                nl.append(c[0])
        return nl


cgt = {}

# --------------------------------------------------------
t = CGK.CGNSLibraryVersion_ts
cgt[t] = CGNStype(t, dtype=[CGK.R4], names=[CGK.CGNSLibraryVersion_s])
cgt[t].shape = (1,)

# (CGK.CGNSLibraryVersion_ts,   # SIDS type
# [CGK.CGNSLibraryVersion_s],  # Names
# [CGK.R4],                    # Datatypes
# (1,),                        # Shape
# [2.3,2.4,3.2,3.3],           # Values
# )

# --------------------------------------------------------
t = CGK.Descriptor_ts
cgt[t] = CGNStype(t, dtype=[CGK.C1])
cgt[t].shape = (0,)

# --------------------------------------------------------
t = CGK.Ordinal_ts
cgt[t] = CGNStype(t, dtype=[CGK.I4], names=[CGK.Ordinal_s])
cgt[t].shape = (1,)

# --------------------------------------------------------
t = CGK.DataClass_ts
cgt[t] = CGNStype(t, dtype=[CGK.C1], names=[CGK.DataClass_s])
cgt[t].shape = (0,)
cgt[t].enumerate = CGK.DataClass_l

# --------------------------------------------------------
t = CGK.DimensionalUnits_ts
cgt[t] = CGNStype(t, dtype=[CGK.C1], names=[CGK.DimensionalUnits_s])
cgt[t].shape = (32, 5)
cgt[t].enumerate = CGK.AllDimensionalUnits_l
cgt[t].addChild(CGK.AdditionalUnits_ts, CGK.AdditionalUnits_s)

# --------------------------------------------------------
t = CGK.AdditionalUnits_ts
cgt[t] = CGNStype(t, dtype=[CGK.C1], names=[CGK.AdditionalUnits_s])
cgt[t].shape = (32, 3)
cgt[t].enumerate = CGK.AllAdditionalUnits_l

# --------------------------------------------------------
t = CGK.DataConversion_ts
cgt[t] = CGNStype(t, dtype=[CGK.R4, CGK.R8], names=[CGK.DataConversion_s])
cgt[t].shape = (2,)

# --------------------------------------------------------
t = CGK.DimensionalExponents_ts
cgt[t] = CGNStype(t, dtype=[CGK.R4, CGK.R8], names=[CGK.DimensionalExponents_s])
cgt[t].shape = (5,)

# --------------------------------------------------------
t = CGK.AdditionalExponents_ts
cgt[t] = CGNStype(t, dtype=[CGK.R4, CGK.R8], names=[CGK.AdditionalExponents_s])
cgt[t].shape = (3,)

# --------------------------------------------------------
t = CGK.DataArray_ts
cgt[t] = CGNStype(t, dtype=allDT)
cgt[t].addChild(CGK.DimensionalExponents_ts, CGK.DimensionalExponents_s)
cgt[t].addChild(CGK.DataConversion_ts, CGK.DataConversion_s)
cgt[t].addChild(CGK.DataClass_ts, CGK.DataClass_s)
cgt[t].addChild(CGK.Descriptor_ts)
cgt[t].addChild(CGK.DimensionalUnits_ts, CGK.DimensionalUnits_s)

# --------------------------------------------------------
t = CGK.Transform_ts
cgt[t] = CGNStype(t, dtype=[CGK.I4], names=[CGK.Transform_s])
cgt[t].shape = (0,)
t = CGK.Transform_ts2
cgt[t] = CGNStype(t, dtype=[CGK.I4], names=[CGK.Transform_s])
cgt[t].shape = (0,)

# --------------------------------------------------------
t = CGK.DiffusionModel_ts
cgt[t] = CGNStype(t, dtype=[CGK.I4], names=[CGK.DiffusionModel_s])
cgt[t].shape = (0,)
t = CGK.DiffusionModel_ts2
cgt[t] = CGNStype(t, dtype=[CGK.I4], names=[CGK.DiffusionModel_s])
cgt[t].shape = (0,)

# --------------------------------------------------------
t = CGK.InwardNormalIndex_ts
cgt[t] = CGNStype(t, dtype=[CGK.I4], names=[CGK.InwardNormalIndex_s])
cgt[t].shape = (0,)
t = CGK.InwardNormalIndex_ts2
cgt[t] = CGNStype(t, dtype=[CGK.I4], names=[CGK.InwardNormalIndex_s])
cgt[t].shape = (0,)

# --------------------------------------------------------
t = CGK.EquationDimension_ts
cgt[t] = CGNStype(t, dtype=[CGK.I4], names=[CGK.EquationDimension_s])
cgt[t].shape = (1,)
t = CGK.EquationDimension_ts2
cgt[t] = CGNStype(t, dtype=[CGK.I4], names=[CGK.EquationDimension_s])
cgt[t].shape = (1,)

# --------------------------------------------------------
t = CGK.GridLocation_ts
cgt[t] = CGNStype(t, dtype=[CGK.C1], names=[CGK.GridLocation_s])
cgt[t].shape = (0,)

# --------------------------------------------------------
t = CGK.Rind_ts
cgt[t] = CGNStype(t, dtype=[CGK.I4, CGK.I8], names=[CGK.Rind_s])
cgt[t].shape = (0,)

# --------------------------------------------------------
t = CGK.IndexRange_ts
cgt[t] = CGNStype(t, dtype=[CGK.I4, CGK.I8])
cgt[t].shape = (0, 2)
cgt[t].names = [CGK.PointRange_s, CGK.PointRangeDonor_s, CGK.ElementRange_s, UD]

# --------------------------------------------------------
t = CGK.IndexArray_ts
cgt[t] = CGNStype(t, dtype=[CGK.I4, CGK.I8, CGK.R4, CGK.R8])
cgt[t].shape = (0, 0)
cgt[t].names = [CGK.PointList_s, CGK.PointListDonor_s, CGK.CellListDonor_s,
                CGK.InwardNormalList_s, UD]

# --------------------------------------------------------
t = CGK.ReferenceState_ts
cgt[t] = CGNStype(t, names=[CGK.ReferenceState_s])
cgt[t].addChild(CGK.Descriptor_ts)
cgt[t].addChild(CGK.Descriptor_ts, CGK.ReferenceStateDescription_s)
cgt[t].addChild(CGK.DataClass_ts, CGK.DataClass_s)
cgt[t].addChild(CGK.DimensionalUnits_ts, CGK.DimensionalUnits_s)
cgt[t].addChild(CGK.DataArray_ts)
cgt[t].addChild(CGK.UserDefinedData_ts)

# --------------------------------------------------------
t = CGK.ConvergenceHistory_ts
cgt[t] = CGNStype(t, names=[CGK.GlobalConvergenceHistory_s,
                            CGK.ZoneConvergenceHistory_s], dtype=[CGK.I4])
cgt[t].shape = (1,)
cgt[t].addChild(CGK.Descriptor_ts)
cgt[t].addChild(CGK.Descriptor_ts, CGK.NormDefinitions_s)
cgt[t].addChild(CGK.DataClass_ts, CGK.DataClass_s)
cgt[t].addChild(CGK.DimensionalUnits_ts, CGK.DimensionalUnits_s)
cgt[t].addChild(CGK.DataArray_ts)
cgt[t].addChild(CGK.UserDefinedData_ts)

# --------------------------------------------------------
t = CGK.IntegralData_ts
cgt[t] = CGNStype(t)
cgt[t].addChild(CGK.Descriptor_ts)
cgt[t].addChild(CGK.DataClass_ts, CGK.DataClass_s)
cgt[t].addChild(CGK.DimensionalUnits_ts, CGK.DimensionalUnits_s)
cgt[t].addChild(CGK.DataArray_ts)
cgt[t].addChild(CGK.UserDefinedData_ts)

# --------------------------------------------------------
t = CGK.UserDefinedData_ts
cgt[t] = CGNStype(t)
cgt[t].addChild(CGK.Descriptor_ts)
cgt[t].addChild(CGK.GridLocation_ts, CGK.GridLocation_s)
cgt[t].addChild(CGK.IndexRange_ts, CGK.PointRange_s)
cgt[t].addChild(CGK.IndexArray_ts, CGK.PointList_s)
cgt[t].addChild(CGK.DataClass_ts, CGK.DataClass_s)
cgt[t].addChild(CGK.DimensionalUnits_ts, CGK.DimensionalUnits_s)
cgt[t].addChild(CGK.DataArray_ts)
cgt[t].addChild(CGK.FamilyName_ts, [CGK.FamilyName_s], card=C_01)
cgt[t].addChild(CGK.AdditionalFamilyName_ts, card=C_0N)
cgt[t].addChild(CGK.UserDefinedData_ts)
cgt[t].addChild(CGK.Ordinal_ts, CGK.Ordinal_s)

# --------------------------------------------------------
t = CGK.Gravity_ts
cgt[t] = CGNStype(t)
cgt[t].addChild(CGK.DataArray_ts, CGK.GravityVector_s)
cgt[t].addChild(CGK.Descriptor_ts)
cgt[t].addChild(CGK.DataClass_ts, CGK.DataClass_s)
cgt[t].addChild(CGK.DimensionalUnits_ts, CGK.DimensionalUnits_s)
cgt[t].addChild(CGK.UserDefinedData_ts)

# --------------------------------------------------------
t = CGK.FlowEquationSet_ts
cgt[t] = CGNStype(t, names=[CGK.FlowEquationSet_s])
cgt[t].addChild(CGK.GoverningEquations_ts, CGK.GoverningEquations_s)
cgt[t].addChild(CGK.EquationDimension_ts, CGK.EquationDimension_s)
cgt[t].addChild(CGK.GasModel_ts, CGK.GasModel_s)
cgt[t].addChild(CGK.ViscosityModel_ts, CGK.ViscosityModel_s)
cgt[t].addChild(CGK.ThermalRelaxationModel_ts, CGK.ThermalRelaxationModel_s)
cgt[t].addChild(CGK.ThermalConductivityModel_ts, CGK.ThermalConductivityModel_s)
cgt[t].addChild(CGK.TurbulenceModel_ts, CGK.TurbulenceModel_s)
cgt[t].addChild(CGK.TurbulenceClosure_ts, CGK.TurbulenceClosure_s)
cgt[t].addChild(CGK.ChemicalKineticsModel_ts, CGK.ChemicalKineticsModel_s)
cgt[t].addChild(CGK.EMMagneticFieldModel_ts, CGK.EMMagneticFieldModel_s)
cgt[t].addChild(CGK.EMElectricFieldModel_ts, CGK.EMElectricFieldModel_s)
cgt[t].addChild(CGK.EMConductivityModel_ts, CGK.EMConductivityModel_s)
cgt[t].addChild(CGK.Descriptor_ts)
cgt[t].addChild(CGK.DataClass_ts, CGK.DataClass_s)
cgt[t].addChild(CGK.DimensionalUnits_ts, CGK.DimensionalUnits_s)
cgt[t].addChild(CGK.UserDefinedData_ts)

# --------------------------------------------------------
t = CGK.GoverningEquations_ts
cgt[t] = CGNStype(t, dtype=[CGK.C1], names=[CGK.GoverningEquations_s])
cgt[t].shape = (0,)
cgt[t].enumerate = CGK.GoverningEquationsType_l
cgt[t].addChild(CGK.Descriptor_ts)
cgt[t].addChild(CGK.DiffusionModel_ts, CGK.DiffusionModel_s)
cgt[t].addChild(CGK.UserDefinedData_ts)

# --------------------------------------------------------
t = CGK.GasModel_ts
cgt[t] = CGNStype(t, dtype=[CGK.C1], names=[CGK.GasModel_s])
cgt[t].shape = (0,)
cgt[t].enumerate = CGK.GasModelType_l
cgt[t].addChild(CGK.Descriptor_ts)
cgt[t].addChild(CGK.DataClass_ts, CGK.DataClass_s)
cgt[t].addChild(CGK.DimensionalUnits_ts, CGK.DimensionalUnits_s)
cgt[t].addChild(CGK.DataArray_ts)
cgt[t].addChild(CGK.UserDefinedData_ts)

# --------------------------------------------------------
t = CGK.ViscosityModel_ts
cgt[t] = CGNStype(t, dtype=[CGK.C1], names=[CGK.ViscosityModel_s])
cgt[t].shape = (0,)
cgt[t].enumerate = CGK.ViscosityModelType_l
cgt[t].addChild(CGK.Descriptor_ts)
cgt[t].addChild(CGK.DataClass_ts, CGK.DataClass_s)
cgt[t].addChild(CGK.DimensionalUnits_ts, CGK.DimensionalUnits_s)
cgt[t].addChild(CGK.DataArray_ts)
cgt[t].addChild(CGK.UserDefinedData_ts)

# --------------------------------------------------------
t = CGK.ThermalConductivityModel_ts
cgt[t] = CGNStype(t, dtype=[CGK.C1], names=[CGK.ThermalConductivityModel_s])
cgt[t].shape = (0,)
cgt[t].enumerate = CGK.ThermalConductivityModelType_l
cgt[t].addChild(CGK.Descriptor_ts)
cgt[t].addChild(CGK.DataClass_ts, CGK.DataClass_s)
cgt[t].addChild(CGK.DimensionalUnits_ts, CGK.DimensionalUnits_s)
cgt[t].addChild(CGK.DataArray_ts)
cgt[t].addChild(CGK.UserDefinedData_ts)

# --------------------------------------------------------
t = CGK.TurbulenceClosure_ts
cgt[t] = CGNStype(t, dtype=[CGK.C1], names=[CGK.TurbulenceClosure_s])
cgt[t].shape = (0,)
cgt[t].enumerate = CGK.TurbulenceClosureType_l
cgt[t].addChild(CGK.Descriptor_ts)
cgt[t].addChild(CGK.DataClass_ts, CGK.DataClass_s)
cgt[t].addChild(CGK.DimensionalUnits_ts, CGK.DimensionalUnits_s)
cgt[t].addChild(CGK.DataArray_ts)
cgt[t].addChild(CGK.UserDefinedData_ts)

# --------------------------------------------------------
t = CGK.TurbulenceModel_ts
cgt[t] = CGNStype(t, names=[CGK.TurbulenceModel_s])
cgt[t].datatype = [CGK.C1]
cgt[t].shape = (0,)
cgt[t].enumerate = CGK.TurbulenceModelType_l
cgt[t].addChild(CGK.Descriptor_ts)
cgt[t].addChild(CGK.DataArray_ts)
cgt[t].addChild(CGK.UserDefinedData_ts)
cgt[t].addChild(CGK.DataClass_ts, CGK.DataClass_s)
cgt[t].addChild(CGK.DimensionalUnits_ts, CGK.DimensionalUnits_s)
cgt[t].addChild(CGK.DiffusionModel_ts, CGK.DiffusionModel_s)

# --------------------------------------------------------
t = CGK.ThermalRelaxationModel_ts
cgt[t] = CGNStype(t, dtype=[CGK.C1], names=[CGK.ThermalRelaxationModel_s])
cgt[t].shape = (0,)
cgt[t].enumerate = CGK.ThermalRelaxationModelType_l
cgt[t].addChild(CGK.Descriptor_ts)
cgt[t].addChild(CGK.DataClass_ts, CGK.DataClass_s)
cgt[t].addChild(CGK.DimensionalUnits_ts, CGK.DimensionalUnits_s)
cgt[t].addChild(CGK.DataArray_ts)
cgt[t].addChild(CGK.UserDefinedData_ts)

# --------------------------------------------------------
t = CGK.ChemicalKineticsModel_ts
cgt[t] = CGNStype(t, dtype=[CGK.C1], names=[CGK.ChemicalKineticsModel_s])
cgt[t].shape = (0,)
cgt[t].enumerate = CGK.ChemicalKineticsModelType_l
cgt[t].addChild(CGK.Descriptor_ts)
cgt[t].addChild(CGK.DataClass_ts, CGK.DataClass_s)
cgt[t].addChild(CGK.DimensionalUnits_ts, CGK.DimensionalUnits_s)
cgt[t].addChild(CGK.DataArray_ts)
cgt[t].addChild(CGK.UserDefinedData_ts)

# --------------------------------------------------------
t = CGK.EMElectricFieldModel_ts
cgt[t] = CGNStype(t, dtype=[CGK.C1], names=[CGK.EMElectricFieldModel_s])
cgt[t].shape = (0,)
cgt[t].enumerate = CGK.EMElectricFieldModelType_l
cgt[t].addChild(CGK.Descriptor_ts)
cgt[t].addChild(CGK.DataClass_ts, CGK.DataClass_s)
cgt[t].addChild(CGK.DimensionalUnits_ts, CGK.DimensionalUnits_s)
cgt[t].addChild(CGK.DataArray_ts)
cgt[t].addChild(CGK.UserDefinedData_ts)

# --------------------------------------------------------
t = CGK.EMMagneticFieldModel_ts
cgt[t] = CGNStype(t, dtype=[CGK.C1], names=[CGK.EMMagneticFieldModel_s])
cgt[t].shape = (0,)
cgt[t].enumerate = CGK.EMMagneticFieldModelType_l
cgt[t].addChild(CGK.Descriptor_ts)
cgt[t].addChild(CGK.DataClass_ts, CGK.DataClass_s)
cgt[t].addChild(CGK.DimensionalUnits_ts, CGK.DimensionalUnits_s)
cgt[t].addChild(CGK.DataArray_ts)
cgt[t].addChild(CGK.UserDefinedData_ts)

# --------------------------------------------------------
t = CGK.EMConductivityModel_ts
cgt[t] = CGNStype(t, dtype=[CGK.C1], names=[CGK.EMConductivityModel_s])
cgt[t].shape = (0,)
cgt[t].enumerate = CGK.EMConductivityModelType_l
cgt[t].addChild(CGK.Descriptor_ts)
cgt[t].addChild(CGK.DataClass_ts, CGK.DataClass_s)
cgt[t].addChild(CGK.DimensionalUnits_ts, CGK.DimensionalUnits_s)
cgt[t].addChild(CGK.DataArray_ts)
cgt[t].addChild(CGK.UserDefinedData_ts)

# --------------------------------------------------------
t = CGK.ZoneType_ts
cgt[t] = CGNStype(t, dtype=[CGK.C1], names=[CGK.ZoneType_s])
cgt[t].shape = (0,)
cgt[t].enumerate = CGK.ZoneType_l

# --------------------------------------------------------
t = CGK.SimulationType_ts
cgt[t] = CGNStype(t, dtype=[CGK.C1], names=[CGK.SimulationType_s])
cgt[t].shape = (0,)
cgt[t].enumerate = CGK.SimulationType_l

# --------------------------------------------------------
t = CGK.GridConnectivityType_ts
cgt[t] = CGNStype(t, dtype=[CGK.C1], names=[CGK.GridConnectivityType_s])
cgt[t].shape = (0,)
cgt[t].enumerate = CGK.GridConnectivityType_l

# --------------------------------------------------------
t = CGK.Family_ts
cgt[t] = CGNStype(t)
cgt[t].addChild(CGK.Descriptor_ts)
cgt[t].addChild(CGK.Ordinal_ts, CGK.Ordinal_s)
cgt[t].addChild(CGK.FamilyBC_ts, card=C_01)
cgt[t].addChild(CGK.GeometryReference_ts)
cgt[t].addChild(CGK.RotatingCoordinates_ts, CGK.RotatingCoordinates_s)
cgt[t].addChild(CGK.FamilyName_ts, card=C_0N)
cgt[t].addChild(CGK.UserDefinedData_ts)

# --------------------------------------------------------
t = CGK.FamilyName_ts
cgt[t] = CGNStype(t, dtype=[CGK.C1], names=[CGK.FamilyName_s])
cgt[t].shape = (0,)

# --------------------------------------------------------
t = CGK.AdditionalFamilyName_ts
cgt[t] = CGNStype(t, dtype=[CGK.C1])
cgt[t].shape = (0,)

# --------------------------------------------------------
t = CGK.FamilyBC_ts
cgt[t] = CGNStype(t, dtype=[CGK.C1], names=[CGK.FamilyBC_s])
cgt[t].shape = (0,)
cgt[t].enumerate = CGK.BCType_l
cgt[t].addChild(CGK.FamilyBCDataSet_ts)

# --------------------------------------------------------
t = CGK.FamilyBCDataSet_ts
cgt[t] = CGNStype(t, dtype=[CGK.C1])
cgt[t].enumerate = CGK.BCTypeSimple_l
cgt[t].addChild(CGK.BCData_ts, CGK.NeumannData_s)
cgt[t].addChild(CGK.BCData_ts, CGK.DirichletData_s)
cgt[t].addChild(CGK.Descriptor_ts)
cgt[t].addChild(CGK.ReferenceState_ts, CGK.ReferenceState_s)
cgt[t].addChild(CGK.DataClass_ts, CGK.DataClass_s)
cgt[t].addChild(CGK.DimensionalUnits_ts, CGK.DimensionalUnits_s)
cgt[t].addChild(CGK.UserDefinedData_ts)

# --------------------------------------------------------
t = CGK.GeometryReference_ts
cgt[t] = CGNStype(t)
cgt[t].addChild(CGK.Descriptor_ts)
cgt[t].addChild(CGK.GeometryFile_ts, CGK.GeometryFile_s)
cgt[t].addChild(CGK.GeometryFormat_ts, CGK.GeometryFormat_s)
cgt[t].addChild(CGK.GeometryEntity_ts)
cgt[t].addChild(CGK.UserDefinedData_ts)

# --------------------------------------------------------
t = CGK.GeometryFile_ts
cgt[t] = CGNStype(t, dtype=[CGK.C1], names=[CGK.GeometryFile_s])
cgt[t].shape = (0,)

# --------------------------------------------------------
t = CGK.GeometryFormat_ts
cgt[t] = CGNStype(t, dtype=[CGK.C1], names=[CGK.GeometryFormat_s])
cgt[t].shape = (0,)

# --------------------------------------------------------
t = CGK.GeometryEntity_ts
cgt[t] = CGNStype(t)

# --------------------------------------------------------
t = CGK.CGNSTree_ts
cgt[t] = CGNStype(t, names=[CGK.CGNSTree_s, UD])
cgt[t].addChild(CGK.CGNSLibraryVersion_ts, [CGK.CGNSLibraryVersion_s], card=C_11)
cgt[t].addChild(CGK.CGNSBase_ts, card=C_0N)

# --------------------------------------------------------
t = CGK.CGNSBase_ts
cgt[t] = CGNStype(t, dtype=[CGK.I4])
cgt[t].shape = (0, 0)
cgt[t].addChild(CGK.Zone_ts, card=C_0N)
cgt[t].addChild(CGK.SimulationType_ts, [CGK.SimulationType_s], card=C_01)
cgt[t].addChild(CGK.BaseIterativeData_ts, card=C_01)
cgt[t].addChild(CGK.IntegralData_ts, card=C_0N)
cgt[t].addChild(CGK.ConvergenceHistory_ts, [CGK.GlobalConvergenceHistory_s], card=C_01)
cgt[t].addChild(CGK.Family_ts, card=C_0N)
cgt[t].addChild(CGK.FlowEquationSet_ts, [CGK.FlowEquationSet_s], card=C_01)
cgt[t].addChild(CGK.ReferenceState_ts, [CGK.ReferenceState_s], card=C_01)
cgt[t].addChild(CGK.Axisymmetry_ts, [CGK.Axisymmetry_s], card=C_01)
cgt[t].addChild(CGK.RotatingCoordinates_ts, [CGK.RotatingCoordinates_s], card=C_01)
cgt[t].addChild(CGK.Gravity_ts, [CGK.Gravity_s], card=C_01)
cgt[t].addChild(CGK.DataClass_ts, [CGK.DataClass_s], card=C_01)
cgt[t].addChild(CGK.DimensionalUnits_ts, [CGK.DimensionalUnits_s], card=C_01)
cgt[t].addChild(CGK.Descriptor_ts, card=C_0N)
cgt[t].addChild(CGK.UserDefinedData_ts, card=C_0N)

# --------------------------------------------------------
t = CGK.Zone_ts
cgt[t] = CGNStype(t, dtype=[CGK.I4, CGK.I8])
cgt[t].shape = (0, 3)
cgt[t].addChild(CGK.GridCoordinates_ts, card=C_0N)
cgt[t].addChild(CGK.DiscreteData_ts, card=C_0N)
cgt[t].addChild(CGK.Elements_ts, card=C_0N)
cgt[t].addChild(CGK.ZoneBC_ts, CGK.ZoneBC_s, card=C_01)
cgt[t].addChild(CGK.FlowSolution_ts, card=C_0N)
cgt[t].addChild(CGK.ZoneSubRegion_ts, card=C_0N)
cgt[t].addChild(CGK.ZoneType_ts, CGK.ZoneType_s, card=C_11)
cgt[t].addChild(CGK.Ordinal_ts, CGK.Ordinal_s, card=C_01)
cgt[t].addChild(CGK.ZoneGridConnectivity_ts, CGK.ZoneGridConnectivity_s, card=C_01)
cgt[t].addChild(CGK.ZoneIterativeData_ts, card=C_01)
cgt[t].addChild(CGK.RigidGridMotion_ts, card=C_0N)
cgt[t].addChild(CGK.ReferenceState_ts, CGK.ReferenceState_s, card=C_01)
cgt[t].addChild(CGK.IntegralData_ts, card=C_0N)
cgt[t].addChild(CGK.ArbitraryGridMotion_ts, card=C_0N)
cgt[t].addChild(CGK.FamilyName_ts, CGK.FamilyName_s, card=C_01)
cgt[t].addChild(CGK.AdditionalFamilyName_ts, card=C_0N)
cgt[t].addChild(CGK.FlowEquationSet_ts, CGK.FlowEquationSet_s, card=C_01)
cgt[t].addChild(CGK.ConvergenceHistory_ts, CGK.ZoneConvergenceHistory_s, card=C_01)
cgt[t].addChild(CGK.RotatingCoordinates_ts, CGK.RotatingCoordinates_s, card=C_01)
cgt[t].addChild(CGK.DataClass_ts, CGK.DataClass_s, card=C_01)
cgt[t].addChild(CGK.DimensionalUnits_ts, CGK.DimensionalUnits_s, card=C_01)
cgt[t].addChild(CGK.Descriptor_ts, card=C_0N)
cgt[t].addChild(CGK.UserDefinedData_ts, card=C_0N)

# --------------------------------------------------------
t = CGK.GridCoordinates_ts
cgt[t] = CGNStype(t, names=[CGK.GridCoordinates_s, UD])
cgt[t].addChild(CGK.DataArray_ts, card=C_0N)
cgt[t].addChild(CGK.Rind_ts, CGK.Rind_s, card=C_01)
cgt[t].addChild(CGK.DataClass_ts, CGK.DataClass_s, card=C_01)
cgt[t].addChild(CGK.DimensionalUnits_ts, CGK.DimensionalUnits_s, card=C_01)
cgt[t].addChild(CGK.Descriptor_ts, card=C_0N)
cgt[t].addChild(CGK.UserDefinedData_ts, card=C_0N)

# --------------------------------------------------------
t = CGK.ZoneSubRegion_ts
cgt[t] = CGNStype(t, dtype=[CGK.I4])
cgt[t].shape = (1,)
cgt[t].addChild(CGK.GridLocation_ts, CGK.GridLocation_s)
cgt[t].addChild(CGK.IndexRange_ts, CGK.PointRange_s, card=C_01)
cgt[t].addChild(CGK.IndexArray_ts, CGK.PointList_s, card=C_01)
cgt[t].addChild(CGK.FamilyName_ts, CGK.FamilyName_s, card=C_01)
cgt[t].addChild(CGK.AdditionalFamilyName_ts, card=C_0N)
cgt[t].addChild(CGK.DataArray_ts, card=C_0N)
cgt[t].addChild(CGK.Rind_ts, CGK.Rind_s, card=C_01)
cgt[t].addChild(CGK.DataClass_ts, CGK.DataClass_s, card=C_01)
cgt[t].addChild(CGK.DimensionalUnits_ts, CGK.DimensionalUnits_s, card=C_01)
cgt[t].addChild(CGK.Descriptor_ts, [CGK.BCRegionName_s, CGK.GridConnectivityRegionName_s], card=C_0N)
cgt[t].addChild(CGK.UserDefinedData_ts, card=C_0N)

# --------------------------------------------------------
t = CGK.Elements_ts
cgt[t] = CGNStype(t, dtype=[CGK.I4])
cgt[t].shape = (2,)
cgt[t].addChild(CGK.IndexRange_ts, CGK.PointRange_s)
cgt[t].addChild(CGK.IndexArray_ts, CGK.PointList_s)
cgt[t].addChild(CGK.DataArray_ts, CGK.ElementConnectivity_s, card=C_01)
cgt[t].addChild(CGK.DataArray_ts, CGK.ElementStartOffset_s, card=C_01)
cgt[t].addChild(CGK.DataArray_ts, CGK.ParentElements_s, card=C_01)
cgt[t].addChild(CGK.DataArray_ts, CGK.ParentElementsPosition_s, card=C_01)
cgt[t].addChild(CGK.DataArray_ts, CGK.ParentData_s, card=C_01)
cgt[t].addChild(CGK.Rind_ts, CGK.Rind_s, card=C_01)
cgt[t].addChild(CGK.Descriptor_ts, card=C_0N)
cgt[t].addChild(CGK.UserDefinedData_ts, card=C_0N)

# --------------------------------------------------------
t = CGK.Axisymmetry_ts
cgt[t] = CGNStype(t, names=[CGK.Axisymmetry_s])
cgt[t].addChild(CGK.DataArray_ts, CGK.AxisymmetryReferencePoint_s)
cgt[t].addChild(CGK.DataArray_ts, CGK.AxisymmetryAxisVector_s)
cgt[t].addChild(CGK.DataArray_ts, CGK.AxisymmetryAngle_s)
cgt[t].addChild(CGK.DataArray_ts, CGK.CoordinateNames_s)
cgt[t].addChild(CGK.DataClass_ts, CGK.DataClass_s)
cgt[t].addChild(CGK.DimensionalUnits_ts, CGK.DimensionalUnits_s)
cgt[t].addChild(CGK.Descriptor_ts)
cgt[t].addChild(CGK.UserDefinedData_ts)

# --------------------------------------------------------
t = CGK.RotatingCoordinates_ts
cgt[t] = CGNStype(t, names=[CGK.RotatingCoordinates_s])
cgt[t].addChild(CGK.DataArray_ts, CGK.RotationCenter_s)
cgt[t].addChild(CGK.DataArray_ts, CGK.RotationRateVector_s)
cgt[t].addChild(CGK.DataClass_ts, CGK.DataClass_s)
cgt[t].addChild(CGK.DimensionalUnits_ts, CGK.DimensionalUnits_s)
cgt[t].addChild(CGK.Descriptor_ts)
cgt[t].addChild(CGK.UserDefinedData_ts)

# --------------------------------------------------------
t = CGK.FlowSolution_ts
cgt[t] = CGNStype(t)
cgt[t].addChild(CGK.GridLocation_ts, CGK.GridLocation_s)
cgt[t].addChild(CGK.DataArray_ts)
cgt[t].addChild(CGK.Rind_ts, CGK.Rind_s)
cgt[t].addChild(CGK.DataClass_ts, CGK.DataClass_s)
cgt[t].addChild(CGK.DimensionalUnits_ts, CGK.DimensionalUnits_s)
cgt[t].addChild(CGK.Descriptor_ts)
cgt[t].addChild(CGK.UserDefinedData_ts)

# --------------------------------------------------------
t = CGK.DiscreteData_ts
cgt[t] = CGNStype(t)
cgt[t].addChild(CGK.GridLocation_ts, CGK.GridLocation_s)
cgt[t].addChild(CGK.DataArray_ts)
cgt[t].addChild(CGK.Rind_ts, CGK.Rind_s)
cgt[t].addChild(CGK.DataClass_ts, CGK.DataClass_s)
cgt[t].addChild(CGK.DimensionalUnits_ts, CGK.DimensionalUnits_s)
cgt[t].addChild(CGK.Descriptor_ts)
cgt[t].addChild(CGK.UserDefinedData_ts)

# --------------------------------------------------------
t = CGK.ZoneBC_ts
cgt[t] = CGNStype(t, names=[CGK.ZoneBC_s])
cgt[t].addChild(CGK.BC_ts)
cgt[t].addChild(CGK.ReferenceState_ts, CGK.ReferenceState_s)
cgt[t].addChild(CGK.DataClass_ts, CGK.DataClass_s)
cgt[t].addChild(CGK.DimensionalUnits_ts, CGK.DimensionalUnits_s)
cgt[t].addChild(CGK.Descriptor_ts)
cgt[t].addChild(CGK.UserDefinedData_ts)

# --------------------------------------------------------
t = CGK.BCProperty_ts
cgt[t] = CGNStype(t, names=[CGK.BCProperty_s])
cgt[t].addChild(CGK.Descriptor_ts)
cgt[t].addChild(CGK.UserDefinedData_ts)
cgt[t].addChild(CGK.WallFunction_ts, CGK.WallFunction_s)
cgt[t].addChild(CGK.Area_ts, CGK.Area_s)

# --------------------------------------------------------
t = CGK.BCData_ts
cgt[t] = CGNStype(t, names=[CGK.DirichletData_s, CGK.NeumannData_s])
cgt[t].addChild(CGK.DataArray_ts)
cgt[t].addChild(CGK.DataClass_ts, CGK.DataClass_s)
cgt[t].addChild(CGK.DimensionalUnits_ts, CGK.DimensionalUnits_s)
cgt[t].addChild(CGK.Descriptor_ts)
cgt[t].addChild(CGK.UserDefinedData_ts)

# --------------------------------------------------------
t = CGK.BCDataSet_ts
cgt[t] = CGNStype(t, dtype=[CGK.C1])
cgt[t].enumerate = CGK.BCTypeSimple_l
cgt[t].addChild(CGK.BCData_ts, CGK.NeumannData_s)
cgt[t].addChild(CGK.BCData_ts, CGK.DirichletData_s)
cgt[t].addChild(CGK.GridLocation_ts, CGK.GridLocation_s)
cgt[t].addChild(CGK.IndexRange_ts, CGK.PointRange_s)
cgt[t].addChild(CGK.IndexArray_ts, CGK.PointList_s)
cgt[t].addChild(CGK.Descriptor_ts)
cgt[t].addChild(CGK.ReferenceState_ts, CGK.ReferenceState_s)
cgt[t].addChild(CGK.DataClass_ts, CGK.DataClass_s)
cgt[t].addChild(CGK.DimensionalUnits_ts, CGK.DimensionalUnits_s)
cgt[t].addChild(CGK.UserDefinedData_ts)

# --------------------------------------------------------
t = CGK.BC_ts
cgt[t] = CGNStype(t, dtype=[CGK.C1])
cgt[t].enumerate = CGK.BCType_l
cgt[t].shape = (0,)
cgt[t].addChild(CGK.ReferenceState_ts, CGK.ReferenceState_s)
cgt[t].addChild(CGK.DataClass_ts, CGK.DataClass_s)
cgt[t].addChild(CGK.DimensionalUnits_ts, CGK.DimensionalUnits_s)
cgt[t].addChild(CGK.Descriptor_ts)
cgt[t].addChild(CGK.UserDefinedData_ts)
cgt[t].addChild(CGK.Ordinal_ts, CGK.Ordinal_s)
cgt[t].addChild(CGK.FamilyName_ts, CGK.FamilyName_s)
cgt[t].addChild(CGK.AdditionalFamilyName_ts, card=C_0N)
cgt[t].addChild(CGK.IndexArray_ts, CGK.InwardNormalList_s)
cgt[t].addChild(CGK.BCDataSet_ts)
cgt[t].addChild(CGK.InwardNormalIndex_ts, CGK.InwardNormalIndex_s)
cgt[t].addChild(CGK.IndexArray_ts, CGK.ElementList_s)
cgt[t].addChild(CGK.IndexArray_ts, CGK.PointList_s)
cgt[t].addChild(CGK.IndexRange_ts, CGK.ElementRange_s)
cgt[t].addChild(CGK.IndexRange_ts, CGK.PointRange_s)
cgt[t].addChild(CGK.GridLocation_ts, CGK.GridLocation_s)
cgt[t].addChild(CGK.BCProperty_ts, CGK.BCProperty_s)

# --------------------------------------------------------
t = CGK.ArbitraryGridMotionType_ts
cgt[t] = CGNStype(t, dtype=[CGK.C1],
                  names=[CGK.ArbitraryGridMotionType_s])
cgt[t].shape = (0,)
cgt[t].addChild(CGK.DataArray_ts, card=C_0N)
cgt[t].addChild(CGK.GridLocation_ts, CGK.GridLocation_s)
cgt[t].addChild(CGK.Rind_ts, CGK.Rind_s, card=C_01)
cgt[t].addChild(CGK.DataClass_ts, CGK.DataClass_s, card=C_01)
cgt[t].addChild(CGK.DimensionalUnits_ts, CGK.DimensionalUnits_s, card=C_01)
cgt[t].addChild(CGK.Descriptor_ts, card=C_0N)
cgt[t].addChild(CGK.UserDefinedData_ts, card=C_0N)

# --------------------------------------------------------
t = CGK.RigidGridMotionType_ts
cgt[t] = CGNStype(t, dtype=[CGK.C1], names=[CGK.RigidGridMotionType_s])
cgt[t].shape = (0,)

# --------------------------------------------------------
t = CGK.WallFunctionType_ts
cgt[t] = CGNStype(t, dtype=[CGK.C1], names=[CGK.WallFunctionType_s])
cgt[t].shape = (0,)

# --------------------------------------------------------
t = CGK.WallFunction_ts
cgt[t] = CGNStype(t, names=[CGK.WallFunction_s])
cgt[t].addChild(CGK.Descriptor_ts)
cgt[t].addChild(CGK.UserDefinedData_ts)
cgt[t].addChild(CGK.WallFunctionType_ts, CGK.WallFunctionType_s)

# --------------------------------------------------------
t = CGK.AreaType_ts
cgt[t] = CGNStype(t, dtype=[CGK.C1], names=[CGK.AreaType_s])
cgt[t].shape = (0,)

# --------------------------------------------------------
t = CGK.Area_ts
cgt[t] = CGNStype(t, names=[CGK.Area_s])
cgt[t].addChild(CGK.Descriptor_ts)
cgt[t].addChild(CGK.UserDefinedData_ts)
cgt[t].addChild(CGK.AreaType_ts, CGK.AreaType_s, card=C_11)
cgt[t].addChild(CGK.DataArray_ts, CGK.SurfaceArea_s, dtype=CGK.C1, card=C_11)
cgt[t].addChild(CGK.DataArray_ts, CGK.RegionName_s, dtype=CGK.C1, card=C_11)

# --------------------------------------------------------
t = CGK.BaseIterativeData_ts
cgt[t] = CGNStype(t, dtype=[CGK.I4])
cgt[t].shape = (1,)
cgt[t].addChild(CGK.DataClass_ts, [CGK.DataClass_s], card=C_01)
cgt[t].addChild(CGK.DimensionalUnits_ts, [CGK.DimensionalUnits_s], card=C_01)
cgt[t].addChild(CGK.Descriptor_ts)
cgt[t].addChild(CGK.UserDefinedData_ts)
cgt[t].addChild(CGK.DataArray_ts)

# --------------------------------------------------------
t = CGK.ZoneIterativeData_ts
cgt[t] = CGNStype(t)
cgt[t].addChild(CGK.DataClass_ts, [CGK.DataClass_s], card=C_01)
cgt[t].addChild(CGK.DimensionalUnits_ts, [CGK.DimensionalUnits_s], card=C_01)
cgt[t].addChild(CGK.Descriptor_ts)
cgt[t].addChild(CGK.UserDefinedData_ts)
cgt[t].addChild(CGK.DataArray_ts, [CGK.RigidGridMotionPointers_s,
                                   CGK.ArbitraryGridMotionPointers_s,
                                   CGK.FlowSolutionPointers_s,
                                   CGK.ZoneGridConnectivityPointers_s,
                                   CGK.ZoneSubRegionPointers_s])

# --------------------------------------------------------
t = CGK.RigidGridMotion_ts
cgt[t] = CGNStype(t, dtype=[CGK.C1])
cgt[t].shape = (0,)
cgt[t].enumerate = CGK.RigidGridMotionType_l
cgt[t].addChild(CGK.DataClass_ts, [CGK.DataClass_s], card=C_01)
cgt[t].addChild(CGK.DimensionalUnits_ts, [CGK.DimensionalUnits_s], card=C_01)
cgt[t].addChild(CGK.Descriptor_ts)
cgt[t].addChild(CGK.UserDefinedData_ts)
cgt[t].addChild(CGK.DataArray_ts, [CGK.OriginLocation_s,
                                   CGK.RigidRotationAngle_s,
                                   CGK.RigidRotationRate_s,
                                   CGK.RigidVelocity_s])

# --------------------------------------------------------
t = CGK.ArbitraryGridMotion_ts
cgt[t] = CGNStype(t, dtype=[CGK.C1])
cgt[t].shape = (0,)
cgt[t].enumerate = CGK.ArbitraryGridMotionType_l
cgt[t].addChild(CGK.DataClass_ts, CGK.DataClass_s)
cgt[t].addChild(CGK.DimensionalUnits_ts, CGK.DimensionalUnits_s)
cgt[t].addChild(CGK.Descriptor_ts)
cgt[t].addChild(CGK.UserDefinedData_ts)
cgt[t].addChild(CGK.GridLocation_ts, CGK.GridLocation_s)
cgt[t].addChild(CGK.Rind_ts, CGK.Rind_s)
cgt[t].addChild(CGK.DataArray_ts)

# --------------------------------------------------------
t = CGK.ZoneGridConnectivity_ts
cgt[t] = CGNStype(t, names=[CGK.ZoneGridConnectivity_s])
cgt[t].addChild(CGK.GridConnectivity1to1_ts)
cgt[t].addChild(CGK.GridConnectivity_ts)
cgt[t].addChild(CGK.OversetHoles_ts)
cgt[t].addChild(CGK.Descriptor_ts)
cgt[t].addChild(CGK.UserDefinedData_ts)

# --------------------------------------------------------
t = CGK.GridConnectivityProperty_ts
cgt[t] = CGNStype(t, names=[CGK.GridConnectivityProperty_s])
cgt[t].addChild(CGK.Descriptor_ts)
cgt[t].addChild(CGK.UserDefinedData_ts)
cgt[t].addChild(CGK.Periodic_ts, CGK.Periodic_s)
cgt[t].addChild(CGK.AverageInterface_ts, CGK.AverageInterface_s)

# --------------------------------------------------------
t = CGK.Periodic_ts
cgt[t] = CGNStype(t, names=[CGK.Periodic_s])
cgt[t].addChild(CGK.DataClass_ts, CGK.DataClass_s)
cgt[t].addChild(CGK.DimensionalUnits_ts, CGK.DimensionalUnits_s)
cgt[t].addChild(CGK.Descriptor_ts)
cgt[t].addChild(CGK.UserDefinedData_ts)
cgt[t].addChild(CGK.DataArray_ts, CGK.RotationCenter_s)
cgt[t].addChild(CGK.DataArray_ts, CGK.RotationAngle_s)
cgt[t].addChild(CGK.DataArray_ts, CGK.Translation_s)

# --------------------------------------------------------
t = CGK.AverageInterfaceType_ts
cgt[t] = CGNStype(t, dtype=[CGK.C1], names=[CGK.AverageInterfaceType_s])
cgt[t].shape = (0,)

# --------------------------------------------------------
t = CGK.AverageInterface_ts
cgt[t] = CGNStype(t, names=[CGK.AverageInterface_s])
cgt[t].addChild(CGK.Descriptor_ts)
cgt[t].addChild(CGK.UserDefinedData_ts)
cgt[t].addChild(CGK.AverageInterfaceType_ts, CGK.AverageInterfaceType_s)

# --------------------------------------------------------
t = CGK.GridConnectivity1to1_ts
cgt[t] = CGNStype(t, dtype=[CGK.C1])
cgt[t].shape = (0,)
cgt[t].addChild(CGK.Transform_ts, CGK.Transform_s)
cgt[t].addChild(CGK.IntIndexDimension_ts, CGK.Transform_s)
cgt[t].addChild(CGK.Transform_ts2, CGK.Transform_s)
cgt[t].addChild(CGK.IndexRange_ts, CGK.PointRange_s)
cgt[t].addChild(CGK.IndexRange_ts, CGK.PointRangeDonor_s)
cgt[t].addChild(CGK.Ordinal_ts, CGK.Ordinal_s)
cgt[t].addChild(CGK.GridConnectivityProperty_ts, CGK.GridConnectivityProperty_s)
cgt[t].addChild(CGK.Descriptor_ts)
cgt[t].addChild(CGK.UserDefinedData_ts)

# --------------------------------------------------------
t = CGK.GridConnectivityType_ts
cgt[t] = CGNStype(t, dtype=[CGK.C1], names=[CGK.GridConnectivityType_s])
cgt[t].shape = (0,)

# --------------------------------------------------------
t = CGK.GridConnectivity_ts
cgt[t] = CGNStype(t, dtype=[CGK.C1])
cgt[t].shape = (0,)
cgt[t].addChild(CGK.GridLocation_ts, CGK.GridLocation_s)
cgt[t].addChild(CGK.Ordinal_ts, CGK.Ordinal_s)
cgt[t].addChild(CGK.Descriptor_ts)
cgt[t].addChild(CGK.IndexRange_ts, CGK.PointRange_s)
cgt[t].addChild(CGK.IndexArray_ts, CGK.PointList_s)
cgt[t].addChild(CGK.IndexArray_ts, CGK.PointListDonor_s)
cgt[t].addChild(CGK.IndexArray_ts, CGK.CellListDonor_s)
cgt[t].addChild(CGK.GridConnectivityProperty_ts, CGK.GridConnectivityProperty_s)
cgt[t].addChild(CGK.GridConnectivityType_ts, CGK.GridConnectivityType_s)
cgt[t].addChild(CGK.DataArray_ts, CGK.InterpolantsDonor_s)
cgt[t].addChild(CGK.UserDefinedData_ts)

# --------------------------------------------------------
t = CGK.OversetHoles_ts
cgt[t] = CGNStype(t)
cgt[t].addChild(CGK.Descriptor_ts)
cgt[t].addChild(CGK.IndexArray_ts, CGK.PointList_s)
cgt[t].addChild(CGK.GridLocation_ts, CGK.GridLocation_s)
cgt[t].addChild(CGK.IndexRange_ts)
cgt[t].addChild(CGK.UserDefinedData_ts)

# --------------------------------------------------------
types = cgt
tk = list(types)
tk.sort()
for pk in tk:
    for ck in tk:
        if (ck != pk) and (types[pk].hasChild(ck)):
            types[ck].addParent(pk)

# --- reserved names / SIDS types
cgnsnametypes = {
    CGK.Density_s: (CGK.DataArray_ts, None),
    CGK.Pressure_s: (CGK.DataArray_ts, None),
    CGK.Temperature_s: (CGK.DataArray_ts, None),
    CGK.EnergyInternal_s: (CGK.DataArray_ts, None),
    CGK.Enthalpy_s: (CGK.DataArray_ts, None),
    CGK.Entropy_s: (CGK.DataArray_ts, None),
    CGK.EntropyApprox_s: (CGK.DataArray_ts, None),
    CGK.DensityStagnation_s: (CGK.DataArray_ts, None),
    CGK.PressureStagnation_s: (CGK.DataArray_ts, None),
    CGK.TemperatureStagnation_s: (CGK.DataArray_ts, None),
    CGK.EnergyStagnation_s: (CGK.DataArray_ts, None),
    CGK.EnthalpyStagnation_s: (CGK.DataArray_ts, None),
    CGK.EnergyStagnationDensity_s: (CGK.DataArray_ts, None),
}

for pk in tk:
    pkn = types[pk].names
    if pkn != [UD]:
        for curname in pkn:
            if pkn != UD:
                cgnsnametypes[curname] = (types[pk].type, types[pk].enumerate)

# --- last line



#  -------------------------------------------------------------------------
#  pyCGNS - Python package for CFD General Notation System - 
#  See license.txt file in the root directory of this Python module source  
#  -------------------------------------------------------------------------
#
# Get a link list and a search path list and check it, return diag list
#
# A link entry is (target dir, target file, target node, local node, status)
#
# the status is the CHLone (but it is unused by this check):
# S2P_LKOK          link ok
# S2P_LKNOFILE      file not found in search path
# S2P_LKIGNORED     link ignored (unused by this check)
# S2P_LKFILENOREAD  cannot read target file
# S2P_LKNONODE      cannot find target node
#
# Diagnostics are more detailled in this check, diags are associated with
# a link entry, the search path or with the link list, both can have more
# than one diag:
#
# -- per-link
# 100 link is ok
# 101 link is external to the source file
# 102 link is internal to the source file
#
# 200 no source node parent
# 201 no destination node
# 202 cannot have read access to destination arg file
# 203 arg file doesn't exists
# 204 arg file is not a CGNS/HDF5 file
# 205 arg file found in search path but doesn't match with arg dir
# 206 file name contains a relative path
# 207 file name contains an absolute path
# 208 file name contains dangerous characters
#
# -- per-link list
# 300 link list is ok
# 301 has links to the same node
# 302 more than one directory used
#
# 400 has duplicated entries
# 401 has conflicting entries for the same node
# 402 files have different CGNS versions
# 403 files have different name extensions
# 404 some files are found as duplicated in different paths
# 405 unreasonable link depth (more than 3)
# 406 creates an internal loop
# 407 creates an external loop
# 408 mix of relative and plain file names
#
# -- per-search path
# 500 search path is ok
# 501 has unreachable directories
# 502 has duplicated used files in different directories
#
# In the case of node masking or inconsistency, for example if the CGNSBase
# dimensions are not the same, if the ReferenceState is masked... the check
# has to be performed on the resulting merged tree. No diag related to node
# scoping is emitted by the link check
#
# -- Graph construction
# The graph representation requires a unique node identifier as an 1 to N
# index for N nodes. The convention is to start from 1 with CGNSTree_t node
# and then to count nodes during a width-first parse using an alphabetical
# order of the children names.
# Once this index is set a tuple if formed for each node with the file index
# as the first element. If a link is found, the node index is replaced by
# the target file node index. The construction algorithm is:
# * the top file is open and parsed, the node index are set.
# * an internal link is found, actual local file node index is set
# * an external link is found, the target file is added into pending files
#   list, and an entry in with the target path of the node and the node local
#   node index of its is pushed in the pending linked nodes list
# * next pending file is open and parsed the same way
# * when no more pending file exists, all pending linked nodes are processed
#   by adding the target node index as child of the parent entry
# * when no mode pending linked node exists, the graph is built
#
# -- Loop detection
# The graph is parsed with a depth-first algo, if a node index already exists
# in the current node chain of indices, there is a loop...
#
#  -------------------------------------------------------------------------
from __future__ import unicode_literals
from __future__ import print_function
import CGNS.MAP
import CGNS.PAT.cgnsutils      as CGU
import CGNS.PAT.cgnstypes      as CGT
import CGNS.PAT.cgnskeywords   as CGK
import math
import os


def canonicalName(filename):
    fn = os.path.expanduser(os.path.expandvars(filename))
    fn = os.path.realpath(os.path.abspath(fn))
    fn = os.path.normpath(fn)
    return fn


class CGNSGraph(object):
    pending = {}
    fileindex = []
    index = []

    def __init__(self, filename=None):
        if (filename is not None):
            self.parseOneFile(filename)

    def addCanonicalFilename(self, filename):
        fn = canonicalName(filename)
        if (fn not in self.fileindex):
            self.fileindex += [fn]
            return True
        return False

    def filenameIndex(self, filename):
        fn = canonicalName(filename)
        return self.fileindex.index(fn) + 1

    def parseOneFile(self, filename):
        if (not self.addCanonicalFilename(filename)): return
        flags = CGNS.MAP.S2P_NODATA | CGNS.MAP.S2P_FOLLOWLINKS
        (t, l, p) = CGNS.MAP.load(filename, flags, lksearch=['.'])
        idx = self.filenameIndex(filename)
        self.index += CGU.getAllNodesAsWidthFirstIndex(t, idx)
        self.fillLinksList(idx, l)
        for el in l:
            self.parseOneFile(el[0] + '/' + el[1])
        self.solveLinks()

    def fillLinksList(self, idx, l):
        for lk in l:
            fn = canonicalName(lk[0] + os.sep + lk[1])
            print('FILE', fn)
        print(len(self.fileindex))
        for i in self.fileindex:
            print(i)

    def solveLinks(self):
        pass

    def showIndex(self, sort=False):
        if (sort):
            self.index.sort()
        sz = int(math.log10(max([i[1] for i in self.index])) + 1)
        fmt = "%%.2d %%.%dd %%s" % sz
        for e in self.index:
            print(fmt % (e[0], e[1], e[2]))


for i in range(12):
    g = CGNSGraph('/tmp/CHLone-test-008-%.2d.hdf' % i)
    g.showIndex()



#  ---------------------------------------------------------------------------
#  pyCGNS - Python package for CFD General Notation System -
#  See license.txt file in the root directory of this Python module source  
#  ---------------------------------------------------------------------------
#
from __future__ import unicode_literals
import CGNS.PAT.cgnslib      as C
import CGNS.PAT.cgnserrors   as E
import CGNS.PAT.cgnskeywords as K
import numpy             as N

data = C.newZone(None, '{Zone}', N.array([[5, 4, 0], [7, 7, 0], [9, 8, 0]], order='F'))

g1 = C.newGridCoordinates(data, "GridCoordinates")
C.newRigidGridMotion(data, "{RigidGridMotion}")
C.newArbitraryGridMotion(data, "{ArbitraryGridMotion}")
C.newFlowSolution(data, "{FlowSolution}")
C.newDiscreteData(data, "{DiscreteData}")
C.newIntegralData(data, "{IntegralData}")
C.newZoneGridConnectivity(data, "{GridConnectivity}")
C.newBoundary(data, "{BC}", N.array([[0, 0, 0], [0, 0, 0]]))
C.newZoneIterativeData(data, "{ZoneIterativeData}")
C.newReferenceState(data)
C.newRotatingCoordinates(data)
C.newDataClass(data)
C.newDimensionalUnits(data)
C.newFlowEquationSet(data)
C.newConvergenceHistory(data, K.ZoneConvergenceHistory_s)
C.newUserDefinedData(data, '{UserDefinedData}')
C.newDescriptor(data, '{Descriptor}')
C.newOrdinal(data)

status = '6.3'
comment = 'Full SIDS with all optionals'
pattern = [data, status, comment]



#  -------------------------------------------------------------------------
#  pyCGNS.VAL - Python package for CFD General Notation System - VALidater
#  See license.txt file in the root directory of this Python module source  
#  -------------------------------------------------------------------------
#
from __future__ import unicode_literals
import CGNS.PAT.cgnslib as CGL
import CGNS.PAT.cgnsutils as CGU
import CGNS.PAT.cgnskeywords as CGK
import numpy as NPY

TESTS = []

#  -------------------------------------------------------------------------
tag = 'gridconnectivity1to1'
diag = True
vertexsize = 20
cellsize = 7
ntris = 12


def makeCorrectTree():
    T = CGL.newCGNSTree()
    b = CGL.newBase(T, 'Base', 3, 3)
    z1 = CGL.newZone(b, 'Zone1', NPY.array([[5, 4, 0], [7, 6, 0], [5, 4, 0]], order='F'))
    g = CGL.newGridCoordinates(z1, 'GridCoordinates')
    d = CGL.newDataArray(g, CGK.CoordinateX_s, NPY.ones((5, 7, 5), dtype='float64', order='F'))
    d = CGL.newDataArray(g, CGK.CoordinateY_s, NPY.ones((5, 7, 5), dtype='float64', order='F'))
    d = CGL.newDataArray(g, CGK.CoordinateZ_s, NPY.ones((5, 7, 5), dtype='float64', order='F'))
    s = NPY.array([[vertexsize, cellsize, 0]], dtype='int32', order='F')
    z2 = CGU.copyNode(z1, 'Zone2')
    b[2].append(z2)
    z = [z1, z2]
    return (T, b, z)


(T, b, z) = makeCorrectTree()
zgc = CGL.newZoneGridConnectivity(z[0])
gc = CGL.newGridConnectivity1to1(zgc, 'join1_2', 'Zone2', NPY.array([[1, 1], [1, 7], [1, 5]]),
                                 NPY.array([[1, 5], [1, 7], [5, 5]]), NPY.array([+1, +2, +3]))
zgc = CGL.newZoneGridConnectivity(z[1])
gc = CGL.newGridConnectivity1to1(zgc, 'join2_1', 'Zone1', NPY.array([[1, 5], [1, 7], [5, 5]]),
                                 NPY.array([[1, 1], [1, 7], [1, 5]]), NPY.array([+1, +2, +3]))
TESTS.append((tag, T, diag))

#  -------------------------------------------------------------------------
tag = 'gridconnectivity1to1 bad datatype'
diag = False
(T, b, z) = makeCorrectTree()
zgc = CGL.newZoneGridConnectivity(z[0])
gc = CGL.newGridConnectivity1to1(zgc, 'join1_2', 'Zone2', NPY.array([[1, 1], [1, 7], [1, 5]]),
                                 NPY.array([[1, 1], [1, 7], [5, 5]]), NPY.array([+1, +2, +3]))
gc[1] = NPY.array([1], order='F')
TESTS.append((tag, T, diag))

#  -------------------------------------------------------------------------
tag = 'gridconnectivity1to1 absent opposite GridConnectivity'
diag = False
(T, b, z) = makeCorrectTree()
zgc = CGL.newZoneGridConnectivity(z[0])
gc = CGL.newGridConnectivity1to1(zgc, 'join1_2', 'Zone2', NPY.array([[1, 1], [1, 7], [1, 5]]),
                                 NPY.array([[1, 5], [1, 7], [5, 5]]), NPY.array([+1, +2, +3]))
TESTS.append((tag, T, diag))

#  -------------------------------------------------------------------------
tag = 'gridconnectivity1to1 unvalid opposite GridConnectivity'
diag = False
(T, b, z) = makeCorrectTree()
zgc = CGL.newZoneGridConnectivity(z[0])
gc = CGL.newGridConnectivity1to1(zgc, 'join1_2', 'Zone2', NPY.array([[1, 1], [1, 7], [1, 5]]),
                                 NPY.array([[1, 5], [1, 7], [5, 5]]), NPY.array([+1, +2, +3]))
zgc = CGL.newZoneGridConnectivity(z[1])
gc = CGL.newGridConnectivity1to1(zgc, 'join2_1', 'Zone1', NPY.array([[1, 5], [1, 7], [5, 5]]),
                                 NPY.array([[1, 1], [1, 7], [1, 5]]), NPY.array([+1, +2, +3]))
CGU.removeChildByName(gc, CGK.PointRangeDonor_s)
TESTS.append((tag, T, diag))

#  -------------------------------------------------------------------------
tag = 'gridconnectivity1to1 bad shape on Transform'
diag = False
(T, b, z) = makeCorrectTree()
zgc = CGL.newZoneGridConnectivity(z[0])
gc = CGL.newGridConnectivity1to1(zgc, 'join1_2', 'Zone2', NPY.array([[1, 1], [1, 7], [1, 5]]),
                                 NPY.array([[1, 5], [1, 7], [5, 5]]), NPY.array([[+1, +2, +3]]))
zgc = CGL.newZoneGridConnectivity(z[1])
gc = CGL.newGridConnectivity1to1(zgc, 'join2_1', 'Zone1', NPY.array([[1, 5], [1, 7], [5, 5]]),
                                 NPY.array([[1, 1], [1, 7], [1, 5]]), NPY.array([+1, +2, +3]))
TESTS.append((tag, T, diag))

#  -------------------------------------------------------------------------
tag = 'gridconnectivity1to1 bad Transform'
diag = False
(T, b, z) = makeCorrectTree()
zgc = CGL.newZoneGridConnectivity(z[0])
gc = CGL.newGridConnectivity1to1(zgc, 'join1_2', 'Zone2', NPY.array([[1, 1], [1, 7], [1, 5]]),
                                 NPY.array([[1, 5], [1, 7], [5, 5]]), NPY.array([+1, +2, -4]))
zgc = CGL.newZoneGridConnectivity(z[1])
gc = CGL.newGridConnectivity1to1(zgc, 'join2_1', 'Zone1', NPY.array([[1, 5], [1, 7], [5, 5]]),
                                 NPY.array([[1, 1], [1, 7], [1, 5]]), NPY.array([+1, +2, +4]))
TESTS.append((tag, T, diag))

#  -------------------------------------------------------------------------
tag = 'gridconnectivity1to1 several 0 in Transform'
diag = False
(T, b, z) = makeCorrectTree()
zgc = CGL.newZoneGridConnectivity(z[0])
gc = CGL.newGridConnectivity1to1(zgc, 'join1_2', 'Zone2', NPY.array([[1, 1], [1, 7], [1, 5]]),
                                 NPY.array([[1, 5], [1, 7], [5, 5]]), NPY.array([+1, 0, 0]))
zgc = CGL.newZoneGridConnectivity(z[1])
gc = CGL.newGridConnectivity1to1(zgc, 'join2_1', 'Zone1', NPY.array([[1, 5], [1, 7], [5, 5]]),
                                 NPY.array([[1, 1], [1, 7], [1, 5]]), NPY.array([+1, +2, +3]))
TESTS.append((tag, T, diag))

#  -------------------------------------------------------------------------
tag = 'gridconnectivity1to1 not a face'
diag = False
(T, b, z) = makeCorrectTree()
zgc = CGL.newZoneGridConnectivity(z[0])
gc = CGL.newGridConnectivity1to1(zgc, 'join1_2', 'Zone2', NPY.array([[1, 5], [1, 7], [1, 5]]),
                                 NPY.array([[1, 5], [1, 7], [1, 5]]), NPY.array([+1, +2, +3]))
zgc = CGL.newZoneGridConnectivity(z[1])
gc = CGL.newGridConnectivity1to1(zgc, 'join2_1', 'Zone1', NPY.array([[1, 5], [1, 7], [1, 5]]),
                                 NPY.array([[1, 5], [1, 7], [1, 5]]), NPY.array([+1, +2, +3]))
TESTS.append((tag, T, diag))



#  -------------------------------------------------------------------------
#  pyCGNS - Python package for CFD General Notation System -
#  See license.txt file in the root directory of this Python module source  
#  -------------------------------------------------------------------------
#
from __future__ import unicode_literals
TAG = "\n# pyCGNS:"


class cgnsException(Exception):
    def __init__(self, value, msg=None):
        self.value = value
        self.msg = msg

    def set(self, msg):
        self.msg = msg

    def __perr(self, error_id, *tp):
        try:
            msg = TAG + " ERROR [%.3d]- %s" % (error_id, errorTable[error_id])
        except (TypeError, KeyError):
            msg = TAG + " ERROR [%.3d]- %s" % (error_id, errorTable[999])
        ret = msg
        if tp:
            if isinstance(tp, tuple) and (len(tp) > 0):
                ret = msg % tp
            else:
                ret = msg % tp
        return ret

    def __str__(self):
        if self.msg:
            ret = self.__perr(self.value, self.msg)
        else:
            ret = self.__perr(self.value)
        return ret


# -----------------------------------------------------------------------------
errorTable = {
    0: "No error",
    1: "Node is empty !",
    2: "Node should be a list of <name, value, children, type>",
    3: "Node name should be a string",
    4: "Node [%s] children list should be a list",
    5: "More than one CGNSLibraryVersion node found",
    6: "Parent node for [%s] is node a CGNS node",
    7: "Failed to add a badly formed child node into [%s]",
    8: "Failed to add root node as child node into [%s]",
    9: "Link chaser returns a badly formed node as place of [%s]",
    10: "CGNSBase [%s] bad cell dimensions",
    11: "CGNSBase [%s] bad physical dimensions",
    12: "CGNSBase [%s] bad cell/physical dimensions values",
    20: "No node of type [%s] with name [%s]",
    21: "No node with name [%s]",
    22: "Node name should have type string",
    23: "Empty string is not allowed for a node name",
    24: "Node name should not contain a '/'",
    25: "Node name length should not be greater than 32 chars",
    26: "Bad name [%s] for this node",
    27: "Bad type [%s] for this node",
    28: "Node [%s] should have no child",
    100: "Absolute path implies a root node with CGNSLibraryVersion node",
    101: "No child on CGNS tree first level",
    102: "Duplicated child name [%s] in [%s]",
    103: "Parent node of [%s] should have type [%s]",
    104: "Parent node of [%s] should have type in %s",
    105: "String data should have type array or string",
    106: "Value should be a real array",
    107: "Value should be an integer array",
    108: "Parent node should have type [%s]",
    109: "Value should be an array",
    110: "Value should be a string in [%s]",
    200: "Bad GridLocation value [%s]",
    201: "Bad ConvergenceHistory name [%s]",
    202: "Units specification list should have 5 values (one per unit)",
    203: "Bad units specification [%s]",
    204: "Units specifications duplicated [%s]",
    205: "Bad SimulationType value [%s]",
    206: "Bad ZoneType value [%s]",
    207: "Bad DataClass value [%s]",
    208: "Exponents specification list should have 5 values (one per unit)",
    209: "BaseIterativeData number of steps should be an integer value",
    210: "BaseIterativeData steps should in %s",
    211: "BaseIterativeData value has greater length than number of steps",
    212: "BaseIterativeData is missing sub-node",
    213: "Parent node should be BaseIterativeData",
    214: "BaseIterativeData bad Pointer name [%s]",
    215: "BaseIterativeData Pointer list [%s] has not the right length [%s]",
    216: "BaseIterativeData Pointer list [%s] contents has bad length [%s]",
    217: "Parent node should be EMConductivityModel",
    218: "Bad EMConductivityModelType value [%s]",
    219: "EMConductivityModelType is missing sub-node",
    220: "Parent node should be GoverningEquations",
    221: "Bad GoverningEquationsType value [%s]",
    222: "GoverningEquationsType is missing sub-node",
    223: "Parent node should be GasModel",
    224: "Bad GasModelType value [%s]",
    225: "GasModelType is missing sub-node",
    226: "Parent node should be ThermalConductivityModel",
    227: "Bad ThermalConductivityModelType value [%s]",
    228: "ThermalConductivityModelType is missing sub-node",
    229: "Parent node should be ViscosityModel",
    230: "Bad ViscosityModelType value [%s]",
    231: "ViscosityModelType is missing sub-node",
    232: "Parent node should be TurbulenceClosure",
    233: "Bad TurbulenceClosureType value [%s]",
    234: "TurbulenceClosureType is missing sub-node",
    235: "Parent node should be TurbulenceModel",
    236: "Bad TurbulenceModelType value [%s]",
    237: "TurbulenceModelType is missing sub-node",
    238: "Parent node should be ThermalRelaxationModel",
    239: "Bad ThermalRelaxationModelType value [%s]",
    240: "ThermalRelaxationModelType is missing sub-node",
    241: "Parent node should be ChemicalKineticsModel",
    242: "Bad ChemicalKineticsModelType value [%s]",
    243: "ChemicalKineticsModelType is missing sub-node",
    244: "Parent node should be EMElectricFieldModel",
    245: "Bad EMElectricFieldModelType value [%s]",
    246: "EMElectricFieldModelType is missing sub-node",
    247: "Parent node should be EMMagneticFieldModel",
    248: "Bad EMMagneticFieldModelType value [%s]",
    249: "EMMagneticFieldModelType is missing sub-node",
    250: "Bad ElementType value [%s]",
    251: "Bad ElementConnectivity value [%s]",
    252: "Bad BCTypeSimple value [%s]",
    253: "Bad AverageInterfaceType value [%s]",
    254: "Bad RigidGridMotionType value [%s]",
    255: "Bad ArbitraryGridMotionType value [%s]",
    256: "Bad GeometryFormat value [%s]",
    257: "Bad BCTypeSimple or BCTypeCompound value [%s]",
    258: "Parent node should be ArbitraryGridMotion_t",
    259: "Parent node should be ReferenceState_t",
    260: "Parent node should be IntegralData_t",
    261: "Parent node should be UserDefinedData_t",
    262: "Parent node should be Family_t",
    263: "Parent node should be GridCoordinates_t",
    264: "Parent node should be DiscreteData_t",
    265: "Parent node should be BCData_t",
    266: "Parent node should be ZoneIterativeData_t",
    267: "Parent node should be AverageInterface_t",
    268: "Bad AverageInterfaceType value [%s]",
    269: "AverageInterfaceType is missing sub-node",
    270: "Parent node should be ZoneGridConnectivity_t",
    271: "Bad ZoneGridConnectivityType value [%s]",
    272: "ZoneGridConnectivityType is missing sub-node",
    273: "Parent node should be DimensionalExponents_t",
    274: "Bad SimulationType value [%s]",
    275: "Parent node should be BCDataSet_t",
    276: "BCDataSetType is missing sub-node",
    277: "Parent node should be ZoneBC_t",
    278: "Parent node should be GridCoordinates_t",
    279: "Parent node should be RotatingCoordinates_t",
    280: "Parent node should be Axisymmetry_t",
    281: "Parent node should be FlowSolution_t ",
    282: "Parent node should be Periodic_s ",
    283: "Parent node should be OversetHoles_t ",
    284: "Parent node should be GridConnectivity1to1_t ",
    285: "Parent node should be GridCoordinates",
    286: "Parent node should be DiscreteData_t",
    287: "Parent node should be DataArray_t",
    288: "Parent node should be Zone_t",
    289: "Parent node should be DataConversion_t",
    290: "Parent node should be GridLocation_t",
    291: "Parent node should be Descriptor_t",
    292: "Parent node should be GeometryReference_t",
    293: "GeometryReferenceType is missing sub-node",
    294: "Parent node should be FamilyBC_t",
    295: "FamilyBCType is missing sub-node",

    800: "adf.database_open No such open status [%s]",
    801: "adf.database_open No such open format [%s]",
    802: "adf.database_open Empty file name",

    900: "Cannot find pyCGNS config ?",
    901: "No information about numeric library in pyCGNS config",
    902: "Bad information about numeric library in pyCGNS config [%s]",

    999: "Unknow error code",

    1001: "No such file",
    1002: "Open file failed",
    1003: "File already exists",
    1004: "No file name",
    1005: "Bad ADF status at open",
    1006: "Bad ADF format at open",
    1100: "Not yet connected to a CGNS database",
    1101: "No such base name",
    1102: "Absolute path required",
    1103: "No such zone name",
    1104: "No such array name",
    1105: "No such user data node name",
    1106: "No such flow solution node name",
    1107: "No such boundary condition node name",
    1110: "File should be readable",
    1901: "No such goto path in CGNS tree",
    1200: "Bad type",
    1201: "Bad OpenMode value",
    1202: "Bad ErrorCode type",
    1203: "Bad MassUnit type",
    1204: "Bad LengthUnit type",
    1205: "Bad TimeUnit type",
    1206: "Bad TemperatureUnit type",
    1207: "Bad AngleUnit type",
    1208: "Bad DataClass type",
    1209: "Bad GridLocation type",
    1210: "Bad BC data type",
    1211: "Bad GridConnectivityType type",
    1212: "Bad PointSetType type",
    1213: "Bad GoverningEquation type",
    1214: "Bad Model type",
    1215: "Bad BC type",
    1216: "Bad DataType type",
    1217: "Bad ElementType type",
    1218: "Bad ZoneType type",
    1219: "Bad RigidGridMotionType type",
    1220: "Bad ArbitraryGridMotionType type",
    1221: "Bad SimulationType type",
    1222: "Bad WallFunctionType type",
    1223: "Bad AreaType type",
    1224: "Bad AverageInterfaceType type",

}

CGNS_NoSuchFile = cgnsException(1001)
CGNS_OpenFileFailed = cgnsException(1002)
CGNS_FileAlreadyExists = cgnsException(1003)
CGNS_NoFileName = cgnsException(1004)
CGNS_BadADFstatus = cgnsException(1005)
CGNS_BadADFformat = cgnsException(1006)
CGNS_NotConnected = cgnsException(1100)
CGNS_NoSuchBase = cgnsException(1101)
CGNS_AbsolutePathRequired = cgnsException(1102)
CGNS_NoSuchZone = cgnsException(1103)
CGNS_NoSuchArray = cgnsException(1104)
CGNS_NoSuchUserData = cgnsException(1105)
CGNS_NoSuchFlowSolution = cgnsException(1106)
CGNS_NoSuchBoundary = cgnsException(1107)
CGNS_FileShouldBeReadable = cgnsException(1110)
CGNS_GotoPathNotFound = cgnsException(1901)
CGNS_BadType = cgnsException(1200)
CGNS_BadOpenMode = cgnsException(1201)
CGNS_BadErrorCode = cgnsException(1202)
CGNS_BadMassUnit = cgnsException(1203)
CGNS_BadLengthUnit = cgnsException(1204)
CGNS_BadTimeUnit = cgnsException(1205)
CGNS_BadTemperatureUnit = cgnsException(1206)
CGNS_BadAngleUnit = cgnsException(1207)
CGNS_BadDataClass = cgnsException(1208)
CGNS_BadGridLocation = cgnsException(1209)
CGNS_BadBCDataType = cgnsException(1210)
CGNS_BadGridConnectivityType = cgnsException(1211)
CGNS_BadPointSetType = cgnsException(1212)
CGNS_BadGoverningType = cgnsException(1213)
CGNS_BadModelType = cgnsException(1214)
CGNS_BadBCType = cgnsException(1215)
CGNS_BadDataType = cgnsException(1216)
CGNS_BadElementType = cgnsException(1217)
CGNS_BadZoneType = cgnsException(1218)
CGNS_BadRigidGridMotionType = cgnsException(1219)
CGNS_BadArbitraryGridMotionType = cgnsException(1220)
CGNS_BadSimulationType = cgnsException(1221)
CGNS_BadWallFunctionType = cgnsException(1222)
CGNS_BadAreaType = cgnsException(1223)
CGNS_BadAverageInterfaceType = cgnsException(1224)

# -----------------------------------------------------------------------------
